SystemOrganization addCategory: #'Zinc-HTTP-Core'!
SystemOrganization addCategory: #'Zinc-HTTP-Support'!
SystemOrganization addCategory: #'Zinc-HTTP-Client-Server'!

!String methodsFor: '*zinc-http' stamp: 'lr 8/30/2010 11:22'!
findFirstToken: aToken
	"Finds the first occurance of aToken, and answeres a collection of everything before and after."
	| index tokens |
	(self includesSubString: aToken) ifFalse: [^ self].
	index := self findString: aToken.
	tokens := OrderedCollection new.

	^ tokens
		add: (self copyFrom: 1 to: index - 1);
		add: (self copyFrom: index + aToken size to: self size);
		yourself.! !

!String methodsFor: '*zinc-http' stamp: 'lr 8/30/2010 11:22'!
fromFirstDigit
	| index |
	index := 1.
	[ (self at: index) isDigit ] whileFalse: [ index := index + 1 ].
	
	^ self copyFrom: index to: self size! !

!String methodsFor: '*zinc-http' stamp: 'cc 5/9/2004 19:28'!
quoted
	"Answers the receiver with a copy of self surrounded by double quotes."
	^ $" asString, self, $" asString! !

Stream subclass: #ZnBivalentWriteStream
	instanceVariableNames: 'stream binary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnBivalentWriteStream commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:50' prior: 0!
I am ZnBivalentWriteStream, wrapping another write stream.

I allow both Characters and Strings as well as Integers (0 to: 255) and ByteArrays to be writtten to me.

The write stream that I wrap can be both binary as well as character based.

Part of Zinc HTTP Components.!

!ZnBivalentWriteStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/6/2010 12:49'!
on: writeStream
	"Wrap writeStream as a BivalentWriteStream, unless it is already one"
	
	(writeStream isKindOf: self) ifTrue: [ ^ writeStream ].
	
	^ self basicNew
		on: writeStream;
		yourself! !

!ZnBivalentWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/4/2010 20:07'!
close
	stream close! !

!ZnBivalentWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/3/2010 21:07'!
isBinary
	binary isNil ifTrue: [ 
		[ binary := stream isBinary ] 
			on: MessageNotUnderstood 
			do: [ binary := false ] ].
	^ binary! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:54'!
nextPut: object
	self isBinary
		ifTrue: [ stream nextPut: object asInteger ]
		ifFalse: [ stream nextPut: object asCharacter ]! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:54'!
nextPutAll: collection
	self isBinary
		ifTrue: [ stream nextPutAll: collection asByteArray ]
		ifFalse: [ stream nextPutAll: collection asString ]! !

!ZnBivalentWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/4/2010 20:07'!
on: writeStream
	stream := writeStream! !

!ZnBivalentWriteStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 10/4/2010 20:08'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 20:12'!
space
	self nextPut: Character space! !

Stream subclass: #ZnChunkedReadStream
	instanceVariableNames: 'stream chunk position atEnd totalSize extraHeaders'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnChunkedReadStream commentStamp: 'SvenVanCaekenberghe 9/26/2010 18:15' prior: 0!
ZnChunkedReadStream implements HTTP 1.1 chunked transfer encoding on a wrapped streams.

Clients should read me until I am atEnd.

After I am completely read, I can tell you my totalSize and optional extraHeaders.

Part of Zinc HTTP Components.!

!ZnChunkedReadStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/26/2010 14:10'!
new
	^ super new initialize! !

!ZnChunkedReadStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/26/2010 19:43'!
on: stream
	^ self basicNew
		initialize;
		on: stream;
		yourself! !

!ZnChunkedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:57'!
atEnd
	self ensureChunkOrAtEnd.
	^ atEnd! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 08:11'!
binary
	stream binary! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:45'!
collectionSpecies
	^ self isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]
	! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/26/2010 19:56'!
ensureChunkOrAtEnd
	atEnd 
		ifTrue: [ ^ self ].
	(chunk isNil or: [ position >= chunk size ]) 
		ifTrue: [ self getNextChunk ]
! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:21'!
extraHeaders
	^ extraHeaders! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/26/2010 17:08'!
getExtraHeaders
	extraHeaders := ZnHeaders readFrom: stream! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/26/2010 19:51'!
getNextChunk
	| size |
	(size := self getNextChunkSize) = 0
		ifTrue: [
			atEnd := true.
			chunk := position := nil.
			self getExtraHeaders ]
		ifFalse: [
			totalSize := totalSize + size.
			chunk := self collectionSpecies new: size.
			stream next: size into: chunk.
			stream next: 2. "consume CRLF"
			position := 0 ]! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 19:26'!
getNextChunkSize
	| line |
	(line := (ZnLineReader on: stream) nextLine) isEmpty ifTrue: [ ^ 0 ].
	^ Integer readFrom: (line copyUpTo: $;) base: 16! !

!ZnChunkedReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/26/2010 14:11'!
initialize
	atEnd := false.
	totalSize := 0! !

!ZnChunkedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:45'!
isBinary
	^ stream isBinary
! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:57'!
next
	self ensureChunkOrAtEnd.
	self atEnd ifTrue: [ ^ nil ].
	^ chunk at: (position := position + 1)! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 15:27'!
next: anInteger 
	"We use our own collectionSpecies."

	^ self collectionSpecies streamContents: [ :out | | count |
		count := 0.
		[ self atEnd or: [ count >= anInteger ] ] whileFalse: [
			out nextPut: self next.
			count := count + 1 ] ]! !

!ZnChunkedReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/26/2010 14:13'!
on: readStream
	stream := readStream! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 15:30'!
position
	^ self atEnd 
		ifTrue: [ totalSize ]
		ifFalse: [ totalSize - chunk size + position ]! !

!ZnChunkedReadStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:12'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:12'!
reset
	"We don't allow it"! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 17:10'!
skip: count
	count timesRepeat: [ self next ]! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:20'!
totalSize
	^ totalSize! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 08:12'!
upTo: anObject 
	"We use our own collectionSpecies."
	
	| newStream element |
	newStream := (self collectionSpecies new: 100) writeStream.
	[ self atEnd or: [ (element := self next) = anObject ] ]
		whileFalse: [ newStream nextPut: element ].
	^ newStream contents! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 17:10'!
upToEnd
	"We use out own collectionSpecies."

	| newStream |
	newStream := (self collectionSpecies new: 100) writeStream.
	[ self atEnd ] whileFalse: [ newStream nextPut: self next ].
	^ newStream contents! !

Stream subclass: #ZnLimitedReadStream
	instanceVariableNames: 'stream position limit binary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnLimitedReadStream commentStamp: 'SvenVanCaekenberghe 12/6/2010 13:00' prior: 0!
ZnLimitedReadStream wraps another read stream delegating to it 
but limiting reading to a fixed number of elements.

A ZnLimitedReadStream can be atEnd while its wrapped stream is not.

Our contentSpecies can be forced to be ByteArray (binary) or ByteString (ascii).

We do byte/char conversions on the fly (support bivalent access).

This implementation is really a kludge.

Part of Zinc HTTP Components.!

!ZnLimitedReadStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/15/2010 11:15'!
on: stream limit: limit
	^ self basicNew
		on: stream limit: limit;
		yourself! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 13:38'!
ascii
	binary := false! !

!ZnLimitedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/27/2010 13:07'!
atEnd
	^ position >= limit! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 13:36'!
binary
	binary := true! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 19:34'!
collectionSpecies
	"I can be set to binary or ascii explicitely.
	If not set, delegate to our wrapped stream."
	
	^ binary notNil
		ifTrue: [ 
			binary ifTrue: [ ByteArray ] ifFalse: [ ByteString ] ]
		ifFalse: [
			(stream respondsTo: #collectionSpecies)
				ifTrue: [ stream collectionSpecies ]  
				ifFalse: [ stream isBinary ifTrue: [ ByteArray ] ifFalse: [ ByteString ] ] ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 20:17'!
contentsOfEntireFile
	^ self upToEnd! !

!ZnLimitedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/17/2010 20:06'!
isBinary
	^ binary isNil
		ifTrue: [ stream isBinary ]
		ifFalse: [ binary ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 13:07'!
next
	"We do byte/char conversions on the fly."
	
	^ self atEnd 
		ifTrue: [ 
			nil ] 
		ifFalse: [ 
			position := position + 1. 
			binary isNil
				ifTrue: [ stream next ] 
				ifFalse: [ 
					binary
					 	ifTrue: [ stream next asInteger ]
						ifFalse: [ stream next asCharacter ] ] ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 13:08'!
next: requestedCount 
	"We use our own collectionSpecies."

	| newArray actualCount |
	actualCount := requestedCount min: (limit - position).
	newArray := self collectionSpecies new: actualCount.
	1 to: actualCount do: [ :index | newArray at: index put: self next ].
	^newArray! !

!ZnLimitedReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 13:06'!
on: readStream limit: integer
	stream := readStream.
	limit := integer.
	position := 0! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 13:09'!
position
	^ position! !

!ZnLimitedReadStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/15/2010 12:50'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 17:20'!
reset
	"We don't allow it"! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 20:03'!
skip: count
	count timesRepeat: [ self next ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 13:45'!
upTo: anObject 
	"We use our own collectionSpecies."
	
	| newStream element |
	newStream := (self collectionSpecies new: 100) writeStream.
	[ self atEnd or: [ (element := self next) = anObject ] ]
		whileFalse: [ newStream nextPut: element ].
	^newStream contents! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 13:45'!
upToEnd
	"We use out own collectionSpecies."

	| newStream |
	newStream := (self collectionSpecies new: 100) writeStream.
	[ self atEnd ] whileFalse: [ newStream nextPut: self next ].
	^ newStream contents! !

!HierarchicalUrl methodsFor: '*zinc-http' stamp: 'SvenVanCaekenberghe 9/2/2010 20:04'!
isPathDirectory
	"Is my path a directory without a file component ?"
	
	^ self path last isEmpty! !

!HierarchicalUrl methodsFor: '*zinc-http' stamp: 'SvenVanCaekenberghe 9/2/2010 19:52'!
pathAndQueryString
	^ String streamContents: [ :stream |
		self path do: [ :each |
			stream nextPut: $/.
			stream nextPutAll: each encodeForHTTP ].
		self query isNil ifFalse: [ 
			stream nextPut: $?.
			stream nextPutAll: self query ] ]! !

!HierarchicalUrl methodsFor: '*zinc-http' stamp: 'SvenVanCaekenberghe 9/2/2010 19:59'!
pathDirectoryString
	^ self isPathDirectory 
		ifTrue: [ self pathString ]
		ifFalse: [ String streamContents: [ :stream |
			self path allButLast do: [ :each |
				stream nextPut: $/.
				stream nextPutAll: each encodeForHTTP ].
			stream nextPut: $/ ] ]! !

!HierarchicalUrl methodsFor: '*zinc-http' stamp: 'SvenVanCaekenberghe 9/2/2010 19:52'!
pathString
	^ String streamContents: [ :stream |
		self path do: [ :each |
			stream nextPut: $/.
			stream nextPutAll: each encodeForHTTP ] ]! !

Object subclass: #ZnBasicAuthenticator
	instanceVariableNames: 'credentials realm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnBasicAuthenticator commentStamp: 'SvenVanCaekenberghe 9/30/2010 15:37' prior: 0!
I am ZnBasicAuthenticator.
I help servers handle HTTP Basic Authentication.
I implement the required #authenticateRequest:do:

I have a credentials dictionary with username->password associations.

Part of Zinc HTTP Components.!

!ZnBasicAuthenticator class methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 15:50'!
defaultRealm
	^'ZincHTTPComponents'! !

!ZnBasicAuthenticator class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 15:36'!
new
	^ super new 
		initialize;
		yourself! !

!ZnBasicAuthenticator class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 15:36'!
username: username password: password
	^ self new
		atUsername: username putPassword: password;
		yourself! !

!ZnBasicAuthenticator class methodsFor: 'instance creation' stamp: 'MattKennedy 9/30/2010 15:55'!
username: username password: password realm: realm
	^ self new
		atUsername: username putPassword: password;
		realm: realm;
		yourself! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 17:19'!
atUsername: username putPassword: password
	credentials at: username put: password! !

!ZnBasicAuthenticator methodsFor: 'public' stamp: 'MattKennedy 9/30/2010 15:56'!
authHeader
	^'Basic realm=', self realm! !

!ZnBasicAuthenticator methodsFor: 'public' stamp: 'MattKennedy 9/30/2010 15:56'!
authenticateRequest: request do: block
	^ (self isRequestAuthenticated: request)
		ifTrue: [ block value ]
		ifFalse: [ ZnResponse unauthorized: self authHeader ]! !

!ZnBasicAuthenticator methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/30/2010 15:37'!
initialize
	credentials := Dictionary new! !

!ZnBasicAuthenticator methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/30/2010 15:39'!
isRequestAuthenticated: request
	| authorization username password |
	authorization := (request headers at: 'Authorization' ifAbsent: [ ^ false ]) findTokens: ' '.
	(authorization size = 2 and: [ authorization first = 'Basic' ]) ifFalse: [ ^ false ].
	authorization := authorization second base64Decoded findTokens: ':'.
	authorization size = 2 ifFalse: [ ^ false ].
	username := authorization first.
	password := authorization second.
	^ (credentials at: username ifAbsent: [ nil ]) = password
	! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 15:53'!
realm
	^ realm ifNil: [realm := self class defaultRealm]! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 15:51'!
realm: anObject
	realm := anObject! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 15:41'!
removeAll
	credentials removeAll! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 15:41'!
removeUsername: username
	credentials removeKey: username! !

ZnBasicAuthenticator subclass: #ZnDigestAuthenticator
	instanceVariableNames: 'nonces'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnDigestAuthenticator commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:47' prior: 0!
I am ZnDigestAuthenticator.
I help servers handle HTTP Digest Authentication.

I have a nonces dictionary with nonce->opaque associations. 
I generate a new nonce value for every 401 response I trigger.

Part of Zinc HTTP Components.!

!ZnDigestAuthenticator class methodsFor: 'testing' stamp: 'MattKennedy 10/1/2010 17:36'!
hasMD5Support
	^(self md5Hash: 'test') isNil not! !

!ZnDigestAuthenticator class methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 17:36'!
md5Hash: aString
	| hash |
	"Answer hash of aString as lowercase 32 digit hex String."
	Smalltalk at: #MD5 ifPresent: [:cls |
		^((cls new initialize) hashStream: (aString asByteArray readStream)) hex].
	hash := Smalltalk at: #CMD5Hasher ifPresent: [:cls |
		cls hashMessage: aString].
	hash ifNil: [
		hash := Smalltalk at: #TCryptoRandom ifPresent: [:cls |
			(cls basicNew md5HashMessage: aString) asInteger]].
	hash ifNotNil: [
		hash := hash hex asLowercase.
		(hash beginsWith: '16r') ifTrue: [hash := hash allButFirst: 3].
		hash := hash padded: #left to: 32 with: $0].
	^hash
! !

!ZnDigestAuthenticator class methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 16:59'!
parseAuthRequest: aHeader
	| data dict |
	dict := Dictionary new.
	data := aHeader copyFrom: 8 to: aHeader size.
	(data subStrings: ',') do: [:fragment | | tokens i key value|
		tokens := fragment withBlanksTrimmed.
		i := tokens indexOf: $=.
		(i > 0) ifTrue: [
			key := tokens copyFrom: 1 to: i - 1.
			value := tokens copyFrom: i + 1 to: tokens size.  
			dict at: key put: (value copyFrom: 2 to: value size - 1)]].
	^dict! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'MattKennedy 9/30/2010 17:25'!
a1for: username
	| password |
	password := credentials at: username ifAbsent: [^nil].
	^self class md5Hash: username, ':', self realm, ':', password
	! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'MattKennedy 9/30/2010 17:25'!
a2forUrl: uri method: method
	^self class md5Hash: method, ':', uri
	! !

!ZnDigestAuthenticator methodsFor: 'public' stamp: 'MattKennedy 9/30/2010 16:31'!
authHeader
	| nonce opaque |
	nonce := self createNonce.
	opaque := self createOpaque.
	self nonces at: nonce put: opaque.
	^'Digest realm="', self realm, '", nonce="', nonce, '", opaque="', opaque, '"'! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'MattKennedy 10/1/2010 17:18'!
createNonce
	| pt |
	pt := TimeStamp current asString, ':', Random new next asString.
	^self class md5Hash: pt! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'MattKennedy 10/1/2010 17:21'!
createOpaque
	| pt |
	pt := Random new next asString, ':', TimeStamp current asString.
	^self class md5Hash: pt! !

!ZnDigestAuthenticator methodsFor: 'testing' stamp: 'MattKennedy 9/30/2010 17:25'!
isRequestAuthenticated: request
	| authorization response cresponse a1 a2 nonce opaque |
	authorization := self class parseAuthRequest: (request headers at: 'Authorization' ifAbsent: [ ^ false ]).
	nonce := authorization at: 'nonce' ifAbsent: [^false].
	opaque := self nonces at: nonce ifAbsent: [^false].
	a1 := self a1for: (authorization at: 'username' ifAbsent: [^false]).
	a1 ifNil: [^false].
	a2 := self a2forUrl: (authorization at: 'uri' ifAbsent: ['/']) method: request method.
	response := self class md5Hash: (a1, ':', nonce, ':', a2).
	"(opaque = (authorization at: 'opaque' ifAbsent: [^false])) ifFalse: [^false]."
	cresponse := authorization at: 'response' ifAbsent: [^false].
	^(response = cresponse)! !

!ZnDigestAuthenticator methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 16:16'!
nonces 
	^nonces ifNil: [nonces := Dictionary new]! !

Object subclass: #ZnCharacterEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnCharacterEncoder commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:52' prior: 0!
I am ZnCharacterEncoder, I encode and decode Character objects to and from a binary stream.
I am an abstract class with following protocol:

#nextFromStream:
#nextPut:toStream:
#encodedByteCountFor:

The first two are compatible with TextConverter and subclasses.

Part of Zinc HTTP Components.!

!ZnCharacterEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/29/2010 21:17'!
encodedByteCountFor: character
	"Return how many bytes are needed to encode character"
	
	self subclassResponsibility ! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/29/2010 21:16'!
nextFromStream: stream
	"Read and return the next character from stream"
	
	self subclassResponsibility ! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/29/2010 21:16'!
nextPut: character toStream: stream
	"Write the encoding for character to stream"
	
	self subclassResponsibility ! !

ZnCharacterEncoder subclass: #ZnNullEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnNullEncoder commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:53' prior: 0!
I am ZnNullEncoder, a concrete subclass of ZnCharacterEncoder.
I perform no encoding or decoding at all for all characters with a code value below 256.
I can only be used for ASCII and Latin1 (ISO-8859-1).

Part of Zinc HTTP Components.!

!ZnNullEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/29/2010 21:17'!
encodedByteCountFor: character
	^ 1! !

!ZnNullEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/30/2010 12:03'!
nextFromStream: stream
	^ Character value: stream next! !

!ZnNullEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 12/3/2010 16:52'!
nextPut: character toStream: stream
	| code |
	(code := character charCode) < 256
		ifTrue: [ stream nextPut: code ]
		ifFalse: [ self error: 'Character code outside ISO-8859-1 range' ]! !

ZnCharacterEncoder subclass: #ZnUTF8Encoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnUTF8Encoder commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:55' prior: 0!
I am ZnUTF8Encoder, a concrete subclass of ZnCharacterEncoder.
I implement the variable length UTF-8 encoding and decoding of Unicode according to RFC 3629.

Part of Zinc HTTP Components.!

!ZnUTF8Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/29/2010 21:23'!
encodedByteCountFor: character
	| code |
	code := character charCode.
	code < 128 ifTrue: [ ^ 1 ].
	code < 2048 ifTrue: [ ^ 2 ].
	code < 65535 ifTrue: [ ^ 3 ].
	code <= self maximumUTF8Code ifTrue: [ ^ 4 ].
	self error: 'Unicode character code point outside range'
! !

!ZnUTF8Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/29/2010 21:19'!
maximumUTF8Code
	^ 16r10FFFF! !

!ZnUTF8Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/30/2010 13:46'!
nextFromStream: stream
	| code byte next |
	next := [ | nextByte |
		((nextByte := stream next) bitAnd: 2r11000000) = 2r10000000
			ifTrue: [ code := (code bitShift: 6) + (nextByte bitAnd: 2r00111111) ]
			ifFalse: [ self error: 'illegal UTF-8 encoding' ] ].
	(byte := stream next) < 128 ifTrue: [
		^ Character value: byte ].
	(byte bitAnd: 2r11100000) = 2r11000000 ifTrue: [
		code := byte bitAnd: 2r00011111.
		next value.
		^ Character value: code ].
	(byte bitAnd: 2r11110000) = 2r11100000 ifTrue: [
		code := byte bitAnd: 2r00001111.
		next value; value.
		^ Character value: code ].
	(byte bitAnd: 2r11111000) = 2r11110000 ifTrue: [
		code := byte bitAnd: 2r00000111.
		next value; value; value.
		^ Character value: code ].
	self error: 'illegal UTF-8 encoding'! !

!ZnUTF8Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/30/2010 13:19'!
nextPut: character toStream: stream
	| code next |
	code := character charCode.
	next := [ :shift |
		stream nextPut: (2r10000000 + ((code bitShift: shift) bitAnd: 2r111111)) ].
	code < 128 ifTrue: [ 
		stream nextPut: code.
		^ character ].
	code < 2048 ifTrue: [ 
		stream nextPut: (2r11000000 + (code bitShift: -6)).
		next value: 0.
		^ character ].
	code < 65535 ifTrue: [ 
		stream nextPut: (2r11100000 + (code bitShift: -12)).
		next value: -6; value: 0.
		^ character ].
	code <= self maximumUTF8Code ifTrue: [ 
		stream nextPut: (2r11110000 + (code bitShift: -18)).
		next value: -12; value: -6; value: 0.
		^ character ].
	self error: 'Unicode character code point outside range'
! !

Object subclass: #ZnClient
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnClient commentStamp: 'SvenVanCaekenberghe 12/7/2010 15:18' prior: 0!
I am ZnClient.
I implement the client part of the HTTP protocol. 

My class side implements a number of convenience methods to do common HTTP client operations.

ZnClient get: 'http://caretaker.wolf359.be/small.html'.

We only handle one shot requests.
Have a look at ZnFixedCient and ZnHttpClient for alternatives with more options.

Part of Zinc HTTP Components.!

!ZnClient class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/21/2010 10:43'!
delete: urlObject
	| url stream request response |
	url := urlObject asUrl.
	stream := ZnUtils socketStreamToUrl: url.
	request := ZnRequest delete: url.
	^ self executeOneShot: request on: stream 
	! !

!ZnClient class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/21/2010 10:43'!
delete: urlObject username: username password: password
	| url stream request response |
	url := urlObject asUrl.
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest delete: url)
		setBasicAuthenticationUsername: username password: password.
	^ self executeOneShot: request on: stream 
	! !

!ZnClient class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/6/2010 22:21'!
execute: request on: stream
	request writeOn: stream.
	stream flush.
	^ ZnResponse readFrom: stream! !

!ZnClient class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/21/2010 10:40'!
executeOneShot: request on: stream
	request setConnectionClose.
	^ [ ZnClient execute: request on: stream ] ensure: [ stream close ]! !

!ZnClient class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/15/2010 19:20'!
executeStreaming: request on: stream
	request writeOn: stream.
	stream flush.
	^ ZnResponse readStreamingFrom: stream! !

!ZnClient class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/21/2010 10:40'!
get: urlObject
	"ZnClient get: 'http://caretaker.wolf359.be/small.html'."
	
	| url stream request response |
	url := urlObject asUrl.
	stream := ZnUtils socketStreamToUrl: url.
	request := ZnRequest get: url.
	^ self executeOneShot: request on: stream 
	! !

!ZnClient class methodsFor: 'operations' stamp: 'MattKennedy 9/28/2010 20:56'!
get: urlObject username: username password: password
	| url stream request response |
	url := urlObject asUrl.
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest get: url) 
		setBasicAuthenticationUsername: username password: password.
	^ self executeOneShot: request on: stream 
	! !

!ZnClient class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/13/2010 16:05'!
getGif: urlObject
	"self getGif: 'http://homepage.mac.com/svc/ADayAtTheBeach/calculator.gif'."
	
	^ self 
		getImageOfType: ZnMimeType imageGif 
		usingParser: GIFReadWriter 
		fromUrl: urlObject 
	! !

!ZnClient class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/21/2010 10:42'!
getImageOfType: mimeType usingParser: parserClass fromUrl: urlObject
	| url stream request response |
	url := urlObject asUrl.
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest get: url) 
		setAccept: mimeType.
	response := self executeOneShot: request on: stream.
	response status = 200 ifFalse: [
		^ self error: 'Expected OK response' ].
	response contentType = mimeType ifFalse: [
		^ self error: 'Expected a JPEG' ].
	^ (parserClass on: response entity readStream) nextImage
	! !

!ZnClient class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/13/2010 16:04'!
getJpeg: urlObject
	"ZnClient getJpeg: 'http://caretaker.wolf359.be/sun-fire-x2100.jpg'."
	
	^ self 
		getImageOfType: ZnMimeType imageJpeg 
		usingParser: JPEGReadWriter 
		fromUrl: urlObject 
	! !

!ZnClient class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/13/2010 16:06'!
getPng: urlObject
	"self getPng: 'http://www.pharo-project.org/images/pharo.png'."
	
	^ self 
		getImageOfType: ZnMimeType imagePng 
		usingParser: PNGReadWriter 
		fromUrl: urlObject 
	! !

!ZnClient class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2010 20:50'!
getStreamingImageOfType: mimeType usingParser: parserClass fromUrl: urlObject
	| url stream request response |
	url := urlObject asUrl.
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest get: url) 
		setConnectionClose; 
		setAccept: mimeType.
	response := nil.
	^ [ 
	response := self executeStreaming: request on: stream.
	response status = 200 ifFalse: [
		^ self error: 'Expected OK response' ].
	response contentType = mimeType ifFalse: [
		^ self error: 'Expected a JPEG' ].
	"Actually, the image parsers all just take the whole stream in memory, so the effort is mostly lost"
	(parserClass on: response entity stream) nextImage ]
	ensure: [ response notNil ifTrue: [ response entity consumeContent. stream close ] ]
	! !

!ZnClient class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/21/2010 11:37'!
head: urlObject
	| url stream request response |
	url := urlObject asUrl.
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest head: url)
		setConnectionClose.
	^ [
		request writeOn: stream.
		stream flush.
		ZnResponse readHeaderFrom: stream ] 
	ensure: [ stream close ]	! !

!ZnClient class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/21/2010 12:35'!
head: urlObject username: username password: password
	| url stream request response |
	url := urlObject asUrl.
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest head: url)
		setBasicAuthenticationUsername: username password: password;
		setConnectionClose.
	^ [
		request writeOn: stream.
		stream flush.
		ZnResponse readHeaderFrom: stream ] 
	ensure: [ stream close ]		! !

!ZnClient class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/21/2010 10:54'!
post: urlObject data: entity
	| url stream request response |
	url := urlObject asUrl.
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest post: url)
		entity: entity.
	^ self executeOneShot: request on: stream 
	! !

!ZnClient class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/21/2010 10:55'!
post: urlObject data: entity username: username password: password
	| url stream request response |
	url := urlObject asUrl.
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest post: url)
		entity: entity;
		setBasicAuthenticationUsername: username password: password.
	^ self executeOneShot: request on: stream 
	! !

!ZnClient class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/21/2010 10:55'!
put: urlObject data: entity
	| url stream request response |
	url := urlObject asUrl.
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest put: url)
		entity: entity.
	^ self executeOneShot: request on: stream 
	! !

!ZnClient class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/21/2010 10:56'!
put: urlObject data: entity username: username password: password
	| url stream request response |
	url := urlObject asUrl.
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest put: url)
		entity: entity;
		setBasicAuthenticationUsername: username password: password.
	^ self executeOneShot: request on: stream 
	! !

Object subclass: #ZnConstants
	instanceVariableNames: ''
	classVariableNames: 'HTTPStatusCodes'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnConstants commentStamp: '<historical>' prior: 0!
ZnConstants holds various constants that we use.

Part of Zinc HTTP Components.!

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:18'!
defaultHTTPVersion
	^ 'HTTP/1.1'! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:49'!
defaultUserAgent
	^ self frameworkNameAndVersion ! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:36'!
frameworkLicense
	"http://en.wikipedia.org/wiki/MIT_License"
	
	^ 'MIT License'! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 17:12'!
frameworkMCVersion
	MCWorkingCopy managersForClass: self class do: [ :each | ^ each ancestors first name ]! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:35'!
frameworkName
	^ 'Zinc HTTP Components'! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:47'!
frameworkNameAndVersion
	^ self frameworkName, ' ', self frameworkVersion ! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:36'!
frameworkVersion
	^ '0.1'! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:53'!
httpStatusCodes
	^ HTTPStatusCodes ! !

!ZnConstants class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 9/6/2010 15:51'!
initialize 
	self initializeHTTPStatusCodes! !

!ZnConstants class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/6/2010 16:22'!
initializeHTTPStatusCodes 
	"This is the official list from http://www.iana.org/assignments/http-status-codes"

	HTTPStatusCodes := Dictionary newFromPairs: #(
		100 'Continue'
		101 'Switching Protocols'
		102 'Processing'
		200 'OK'
		201 'Created'
		202 'Accepted'
		203 'Non-Authoritative Information'
		204 'No Content'
		205 'Reset Content'
		206 'Partial Content'
		207 'Multi-Status'
		208 'Already Reported'
		226 'IM Used'
		300 'Multiple Choices'
		301 'Moved Permanently'
		302 'Found'
		303 'See Other'
		304 'Not Modified'
		305 'Use Proxy'
		306 'Reserved'
		307 'Temporary Redirect'
		400 'Bad Request'
		401 'Unauthorized'
		402 'Payment Required'
		403 'Forbidden'
		404 'Not Found'
		405 'Method Not Allowed'
		406 'Not Acceptable'
		407 'Proxy Authentication Required'
		408 'Request Timeout'
		409 'Conflict'
		410 'Gone'
		411 'Length Required'
		412 'Precondition Failed'
		413 'Request Entity Too Large'
		414 'Request-URI Too Long'
		415 'Unsupported Media Type'
		416 'Requested Range Not Satisfiable'
		417 'Expectation Failed'
		422 'Unprocessable Entity'
		423 'Locked'
		424 'Failed Dependency'
		425 'Reserved for WebDAV'
		426 'Upgrade Required'
		500 'Internal Server Error'
		501 'Not Implemented'
		502 'Bad Gateway'
		503 'Service Unavailable'
		504 'Gateway Timeout'
		505 'HTTP Version Not Supported'
		506 'Variant Also Negotiates (Experimental)'
		507 'Insufficient Storage'
		508 'Loop Detected'
		510 'Not Extended')! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:20'!
knownHTTPMethods
	^ #(GET PUT POST DELETE HEAD)! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:22'!
knownHTTPVersions
	^ #('HTTP/1.1' 'HTTP/1.0')! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/9/2010 20:19'!
remoteAddressHeader
	^ 'X-Zinc-Remote-Address'! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:49'!
serverString 
	^ self frameworkNameAndVersion ! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 20:48'!
welcomePageHtml
	^ '<html>
<head>
<title>Zinc HTTP Components</title>
<style type="text/css">
body {
  color: black;
  background: white;
  width: 900px;
  font-family: Verdana, Arial, Helvetica, sans-serif;
  font-size: 13px;
}
p {
  width: 600px;
  padding: 0 20px 10px 20px;
} 
ul,ol {
  width: 600px;
  padding: 0 5px 5px 30px;
} 
#logo {
  color: orange;
  font-family: Helvetica, sans-serif;
  font-weight: bold;
  font-size: 100px;
}
</style>
</head>
<body>
<div id="logo">Zn</div>
<h1>Zinc HTTP Components</h1>
<p>
Welcome to Zinc HTTP Components, 
a modern, open-source Smalltalk framework to deal with the HTTP networking protocol.
</p>
<ul>
<li>Project homepage <a href="http://homepage.mac.com/svc/Zinc-HTTP-Components/">http://homepage.mac.com/svc/Zinc-HTTP-Components/</a></li> 
<li>SqueakSouce page <a href="http://www.squeaksource.com/ZincHTTPComponents.html">http://www.squeaksource.com/ZincHTTPComponents.html</a></li>
</ul>
<h4>May the Source be with you!!</h4>
</body>
</html>', String crlf! !

Object subclass: #ZnCookie
	instanceVariableNames: 'attributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnCookie commentStamp: 'SvenVanCaekenberghe 12/7/2010 00:01' prior: 0!
I represent an HTTP Cookie token.

Part of Zinc HTTP Components.!

!ZnCookie class methodsFor: 'instance creation' stamp: 'MattKennedy 9/28/2010 13:20'!
fromString: aString
	^self new fromString: aString; yourself! !

!ZnCookie class methodsFor: 'instance creation' stamp: 'MattKennedy 9/28/2010 13:48'!
fromString: aString for: urlObject
	| cookie |
	"Ensures a domain value if none provided by the cookie"
	cookie := self new fromString: aString; yourself.
	cookie domain ifNil: [ cookie domain: urlObject authority ].
	^cookie! !

!ZnCookie methodsFor: 'testing' stamp: 'MattKennedy 9/28/2010 10:15'!
= aCookie
	(aCookie isKindOf: self class) ifFalse: [^false].
	^(aCookie name = self name) & (aCookie path = self path) & (aCookie domain = self domain)
	! !

!ZnCookie methodsFor: 'converting' stamp: 'MattKennedy 9/28/2010 13:37'!
asString
	| s |
	s := String new writeStream.
	self writeNameAndValueOn: s.
	^s contents! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:24'!
domain
	^attributes at: 'domain' ifAbsent: [nil]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:25'!
domain: aDomain
	attributes at: 'domain' put: aDomain! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:26'!
expires
	^attributes at: 'expires' ifAbsent: [nil]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:26'!
expires: anExpiration
	attributes at: 'expires' put: anExpiration! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:00'!
expiresTimeStamp
	self expires ifNil: [^ TimeStamp current plusDays: 1].
	^ TimeStamp readFrom: (self expires fromFirstDigit) readStream! !

!ZnCookie methodsFor: 'private' stamp: 'MattKennedy 9/27/2010 20:58'!
extraAttributes
	"Answer all attributes except name and value"
	^attributes associationsSelect: [:ea | (ea key = 'name' or: [ea key = 'value']) not]! !

!ZnCookie methodsFor: 'initialize-release' stamp: 'MattKennedy 9/27/2010 20:38'!
fromString: aString
	| tokens val data i |
	tokens := aString subStrings: ';'.
	i := tokens first indexOf: $=.
	val := tokens first copyFrom: 1 to: i - 1.
	data := tokens first copyFrom: i + 1 to: tokens first size. 
	self name: val; value: (data withBlanksTrimmed).
	2 to: tokens size do: [ :idx | 
		self readAttribute: (tokens at: idx).].! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 21:05'!
httponly
	"This is not an RFC cookie attribute, but a supplement found in MS Internet Explorer and supported in PHP.
	 See http://msdn.microsoft.com/en-us/library/ms533046.aspx"
	^attributes at: 'httponly' ifAbsent: [false]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:28'!
httponly: aBoolean
	attributes at: 'httponly' put: aBoolean! !

!ZnCookie methodsFor: 'initialize-release' stamp: 'MattKennedy 9/27/2010 20:21'!
initialize 
	attributes := Dictionary new.! !

!ZnCookie methodsFor: 'testing' stamp: 'MattKennedy 9/28/2010 10:31'!
isExpired
	^(self expiresTimeStamp asUTC < TimeStamp current asUTC) or: [self expires = 0]! !

!ZnCookie methodsFor: 'testing' stamp: 'MattKennedy 9/28/2010 10:10'!
isInDomain: urlObject
	| domain |
	domain := (self domain beginsWith: '.')
		ifTrue: [ self domain copyFrom: 2 to: self domain size ]
		ifFalse: [ self domain ].
	^urlObject authority includesSubString: domain
	! !

!ZnCookie methodsFor: 'testing' stamp: 'MattKennedy 9/28/2010 10:10'!
isInPath: urlObject
	^urlObject pathString includesSubString: self path
	! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:26'!
maxage
	^self expires! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:22'!
name
	^attributes at: 'name'! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:23'!
name: aName
	attributes at: 'name' put: aName! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:07'!
path
	^attributes at: 'path' ifAbsent: ['/']! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:08'!
path: aPath
	attributes at: 'path' put: aPath! !

!ZnCookie methodsFor: 'initialize-release' stamp: 'MattKennedy 9/28/2010 13:25'!
readAttribute: aToken
	| i key data |
	i := aToken indexOf: $=.
	i > 0
		ifTrue: [
			key := (aToken copyFrom: 1 to: i - 1) withBlanksTrimmed asLowercase.
			data := (aToken copyFrom: i + 1 to: aToken size) withBlanksTrimmed.]
		ifFalse: [ 
			key := (aToken copyFrom: 1 to: aToken size) withBlanksTrimmed asLowercase.
			data := true].
		attributes at: key put: data.! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:27'!
secure
	^attributes at: 'secure' ifAbsent: [false]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:28'!
secure: aBoolean
	attributes at: 'secure' put: aBoolean! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:24'!
value
	^attributes at: 'value' ifAbsent: [nil]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:23'!
value: aValue
	attributes at: 'value' put: aValue! !

!ZnCookie methodsFor: 'printing' stamp: 'MattKennedy 9/28/2010 13:36'!
writeNameAndValueOn: aStream
	aStream nextPutAll: self name, '=', self value! !

!ZnCookie methodsFor: 'printing' stamp: 'MattKennedy 9/27/2010 21:01'!
writeOn: aStream
	aStream nextPutAll: self name, '=', self value.
	self extraAttributes associationsDo: [:ea |
		aStream nextPutAll: '; ', ea key.
		(ea value isKindOf: Boolean) ifFalse: [aStream nextPutAll: '=', ea value]]! !

Object subclass: #ZnCookieJar
	instanceVariableNames: 'cookies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnCookieJar commentStamp: 'SvenVanCaekenberghe 12/7/2010 00:01' prior: 0!
I represent a container for storing HTTP Cookies for a client session.

Part of Zinc HTTP Components.!

!ZnCookieJar methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:37'!
add: aCookie
	(self removeIfExpired: aCookie) ifTrue: [^nil].
	self cookies do: [:cookie|
		cookie = aCookie ifTrue: [ self cookies remove: cookie]].
	self cookies add: aCookie
	! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:12'!
cookieAt: aName forUrl: urlObject
	(self cookiesForUrl: urlObject) do: [:cookie | cookie name = aName ifTrue: [^cookie]].
	^nil! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:24'!
cookies 
	^cookies ifNil: [cookies := Bag new]! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 13:33'!
cookiesForUrl: urlObject
	^self cookies select: [:cookie | 
		(cookie isExpired not) & (cookie isInDomain: urlObject) & (cookie isInPath: urlObject)]! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 13:36'!
remove: aCookie
	self cookies remove: aCookie ifAbsent: [nil]! !

!ZnCookieJar methodsFor: 'operations' stamp: 'MattKennedy 9/28/2010 10:29'!
removeIfExpired: aCookie
	aCookie isExpired ifTrue: [ self remove: aCookie. ^true].
	^false! !

Object subclass: #ZnCredential
	instanceVariableNames: 'domain realm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnCredential commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:56' prior: 0!
I am ZnCredential, an abstract class representing credentials in a domain and realm.

Part of Zinc HTTP Components.!

ZnCredential subclass: #ZnBasicCredential
	instanceVariableNames: 'username password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnBasicCredential commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:58' prior: 0!
I am ZnBasicCredential, a concrete subclass of ZnCredential.
I implement basic HTTP authentication.
I hold a username and password.

Part of Zinc HTTP Components.!

!ZnBasicCredential class methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:28'!
authName
	^'Basic'! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/29/2010 13:49'!
authTokenForUrl: urlObject method: method
	^'Basic ', (username, ':', password) base64Encoded ! !

!ZnBasicCredential methodsFor: 'testing' stamp: 'MattKennedy 10/6/2010 17:17'!
isComplete 
	^username notNil & password notNil ! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
password
	^ password! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
password: anObject
	password := anObject! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
username
	^ username! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
username: anObject
	username := anObject! !

ZnBasicCredential subclass: #ZnDigestCredential
	instanceVariableNames: 'nonce opaque'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnDigestCredential commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:59' prior: 0!
I am ZnDigestCredential, a concrete subclass of ZnCredential and subclass of ZnBasicCredential.
I implement digest HTTP authentication.
I hold a nounce and opaque.

Part of Zinc HTTP Components.!

!ZnDigestCredential class methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:29'!
authName
	^'Digest'! !

!ZnDigestCredential methodsFor: 'computation' stamp: 'MattKennedy 9/30/2010 17:26'!
a1
	^ZnDigestAuthenticator md5Hash: self username, ':', self realm, ':', self password! !

!ZnDigestCredential methodsFor: 'computation' stamp: 'MattKennedy 9/30/2010 17:26'!
a2ForUrl: urlObject method: method
	"This doesn't support all qop options yet."
	^ZnDigestAuthenticator md5Hash: method, ':', (self uriPathFrom: urlObject)! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/29/2010 17:02'!
authTokenForUrl: urlObject method: method
	| result |
	result := String new writeStream.
	result 
		nextPutAll: 'Digest username="', self username, '", ';
		nextPutAll: 'nonce="', self nonce, '", ';
		nextPutAll: 'realm="', self realm, '", ';
		nextPutAll: 'uri="', (self uriPathFrom: urlObject ), '", ';
		nextPutAll: 'response="', (self responseForUrl: urlObject method: method), '", '.
	self opaque isNil ifFalse: [
		result nextPutAll: 'opaque="', self opaque, '"'].
	^result contents! !

!ZnDigestCredential methodsFor: 'testing' stamp: 'MattKennedy 10/6/2010 17:17'!
isComplete 
	^super isComplete & nonce notNil ! !

!ZnDigestCredential methodsFor: 'operations' stamp: 'MattKennedy 9/29/2010 16:25'!
linkToAuthRequest: aHeader
	| data |
	data := self parseAuthRequest: aHeader.
	self nonce: (data at: 'nonce').
	self realm: (data at: 'realm' ifAbsent: [''])! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
nonce
	^ nonce! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
nonce: anObject
	nonce := anObject! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
opaque
	^ opaque! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
opaque: anObject
	opaque := anObject! !

!ZnDigestCredential methodsFor: 'operations' stamp: 'MattKennedy 9/30/2010 16:33'!
parseAuthRequest: aHeader
	^ZnDigestAuthenticator parseAuthRequest: aHeader! !

!ZnDigestCredential methodsFor: 'computation' stamp: 'MattKennedy 9/30/2010 16:40'!
responseForUrl: urlObject method: method
	^ZnDigestAuthenticator md5Hash: self a1, ':', self nonce, ':', (self a2ForUrl: urlObject method: method)! !

!ZnDigestCredential methodsFor: 'operations' stamp: 'MattKennedy 9/29/2010 16:28'!
setAuthorizationHeader: headers for: urlObject method: method
	headers at: 'Authorization' put: (self authTokenForUrl: urlObject method: method)! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/29/2010 16:31'!
uriPathFrom: urlObject
	^urlObject pathString ifNil: ['/']! !

!ZnCredential class methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:28'!
authName
	self subclassResponsibility ! !

!ZnCredential class methodsFor: 'accessing class hierarchy' stamp: 'MattKennedy 9/29/2010 16:19'!
findHandlerFor: anAuthHeader
	| type i |
	i := anAuthHeader indexOf: Character space.
	type := anAuthHeader copyFrom: 1 to: i - 1.
	self allSubclassesDo: [:subclass |
		subclass authName = type ifTrue: [^subclass]].
	^nil! !

!ZnCredential class methodsFor: 'accessing' stamp: 'lr 9/30/2010 08:30'!
findRealmFor: anAuthHeader
	| realm |
	(anAuthHeader subStrings: ',') do: [:token |
		"String>>trimBoth: is a grease method, so not sure if we should be using it."
		(token includesSubString: 'realm=') 
			ifTrue: [ ^((token subStrings: '=') second)]].
	^nil! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/29/2010 16:14'!
authToken
	^self authTokenForUrl: nil method: nil ! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/29/2010 13:48'!
authTokenForUrl: urlObject method: method
	self subclassResponsibility ! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
domain
	^ domain! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
domain: anObject
	domain := anObject! !

!ZnCredential methodsFor: 'testing' stamp: 'MattKennedy 10/6/2010 17:16'!
isComplete
	^self subclassResponsibility ! !

!ZnCredential methodsFor: 'operations' stamp: 'MattKennedy 9/29/2010 16:10'!
linkToAuthRequest: aHeader
	"Link this credential to the WWW-Authenticate request in the provided header. 
	Used for authentication methods which require parameters provided by the server
	(e.g. Digest authentication)."
	realm := self class findRealmFor: aHeader! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 21:45'!
realm
	^ realm ifNil: [realm := '']! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
realm: anObject
	realm := anObject! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/29/2010 13:42'!
setAuthorizationHeader: headers for: urlObject method: method
	headers at: 'Authorization' put: self authToken! !

Object subclass: #ZnEntity
	instanceVariableNames: 'contentType contentLength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnEntity commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:43' prior: 0!
ZnEntity is an abstract HTTP Entity.
An entity has a content type (mime-type), a content length (byte length) and a contents.
Entities can encode and decode themselves to and from binary streams.

Part of Zinc HTTP Components.!

ZnEntity subclass: #ZnApplicationFormUrlEncodedEntity
	instanceVariableNames: 'fields representation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnApplicationFormUrlEncodedEntity commentStamp: '<historical>' prior: 0!
ZnApplicationFormUrlEncodedEntity is a concrete HTTP Entity holding
a set of fields (key value pairs) that is form URL encoded.

Part of Zinc HTTP Components.!

!ZnApplicationFormUrlEncodedEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 20:41'!
designatedMimeType
	^ ZnMimeType applicationFormUrlEncoded ! !

!ZnApplicationFormUrlEncodedEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/13/2010 22:23'!
withAll: keyedCollection
	^ self new
		addAll: keyedCollection;
		yourself ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2010 22:23'!
addAll: keyedCollection
	keyedCollection keysAndValuesDo: [ :key :value |
		self at: key put: value ].
	^ keyedCollection ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/11/2010 21:07'!
at: key
	^ self fields at: key! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/12/2010 10:23'!
at: key ifAbsent: block
	self invalidateRepresentation.
	^ self fields at: key ifAbsent: block! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/12/2010 10:23'!
at: key put: value
	self invalidateRepresentation.
	^ self fields at: key put: value! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/12/2010 10:19'!
computeRepresentation
	representation := String streamContents: [ :stream | self writeRepresentationOn: stream ].
	contentLength := representation size! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/12/2010 10:22'!
contentLength
	contentLength isNil ifTrue: [ self computeRepresentation ].
	^ contentLength ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 16:19'!
contents
	^ self fields! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/11/2010 21:00'!
fields
	^ fields! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/12/2010 10:11'!
initialize
	super initialize.
	fields := Dictionary new.
	self contentType: self class designatedMimeType 
	! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/12/2010 10:17'!
invalidateRepresentation
	representation := nil.
	contentLength := nil! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:41'!
isEmpty
	^ self fields isEmpty! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:42'!
printContentsOn: stream
	super printContentsOn: stream.
	self isEmpty ifFalse: [ 
		fields printElementsOn: stream.
		stream space ]! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/26/2010 19:35'!
readFrom: stream
	"We parse from stream, limited to content lenght if available."
	
	| readStream |
	representation := nil.
	self contentLength isNil
		ifTrue: [ readStream := stream ]
		ifFalse: [ (readStream := ZnLimitedReadStream on: stream limit: self contentLength) ascii ].
	fields := ZnUtils parseQueryFrom: readStream! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/12/2010 10:22'!
writeOn: stream
	"In most use cases, contentLength will have been asked first so that a representation is cached,
	there probably is no need to cache the representation generated here"
	
	representation isNil
		ifTrue: [ self writeRepresentationOn: stream ] 
		ifFalse: [ stream nextPutAll: representation ] ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/13/2010 21:32'!
writeRepresentationOn: stream
	ZnUtils writeQueryFields: self fields on: stream! !

ZnEntity subclass: #ZnByteArrayEntity
	instanceVariableNames: 'bytes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnByteArrayEntity commentStamp: '<historical>' prior: 0!
ZnByteArrayEntity is a concrete HTTP Entity based on a ByteArray.
It is used to hold binary (non textual) data.

Part of Zinc HTTP Components.!

!ZnByteArrayEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/12/2010 11:13'!
bytes: byteArray
	^ (self type: ZnMimeType applicationOctetStream length: byteArray size)
		bytes: byteArray;
		yourself ! !

!ZnByteArrayEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 17:22'!
designatedMimeType
	"I have no designated mime type, I can handle all binary content.
	See #concreteSubclassForType: "
	
	^ nil! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 15:12'!
bytes
	^ bytes! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 15:12'!
bytes: anObject
	bytes := anObject! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2010 22:05'!
contents
	^ self bytes! !

!ZnByteArrayEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:39'!
printContentsOn: stream
	super printContentsOn: stream.
	self bytes notNil ifTrue: [ 
		stream print: self bytes; space ]! !

!ZnByteArrayEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/20/2010 10:37'!
readFrom: stream
	self contentLength isNil
		ifTrue: [ 
			self bytes: stream upToEnd.
			self contentLength: self bytes size ]
		ifFalse: [ | byteArray |
			byteArray := ByteArray ofSize: self contentLength.
			stream next: self contentLength into: byteArray.
			self bytes: byteArray ]! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 19:27'!
readStream
	^ self bytes readStream! !

!ZnByteArrayEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/7/2010 15:12'!
writeOn: stream
	stream nextPutAll: self bytes! !

!ZnEntity class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 13:57'!
byteArrayEntityClass
	^ ZnByteArrayEntity ! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:00'!
bytes: bytes
	^ self byteArrayEntityClass bytes: bytes! !

!ZnEntity class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/3/2010 14:01'!
concreteSubclassForType: mimeType
	"Answer the concrete ZnEntity subclass that handles mimeType"
	
	^ self allSubclasses 
		detect: [ :each | 
			each designatedMimeType = mimeType ]
		ifNone: [ 
			mimeType isBinary 
				ifTrue: [ self byteArrayEntityClass ] 
				ifFalse: [ self stringEntityClass ] ]
! !

!ZnEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 17:22'!
designatedMimeType
	"Answer the mime type we are designated to handle.
	See #concreteSubclassForType: "
	
	self subclassResponsibility ! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:01'!
html: html
	^ self stringEntityClass html: html! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/17/2010 17:07'!
new
	^ super new initialize ! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/15/2010 17:10'!
readFrom: stream usingType: mimeType andLength: length
	"Create and return a new buffered entity of mimeType and length from stream.
	The concrete subclass depends on mimeType"
	
	| newEntity |
	newEntity := (self concreteSubclassForType: mimeType) 
						type: mimeType length: length.
	newEntity readFrom: stream.
	^ newEntity 
! !

!ZnEntity class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/6/2010 13:48'!
stringEntityClass
	^ ZnStringEntity! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:01'!
text: text
	^ self stringEntityClass text: text! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/25/2010 22:16'!
type: type
	"Create a new entity with a given mime type and (currently) undetermined length"
	
	^ self new
		contentType: type;
		yourself! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/12/2010 11:06'!
type: type length: length
	"Create a new entity with a given mime type and length"
	
	^ self new
		contentType: type;
		contentLength: length;
		yourself! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:02'!
with: object
	"Create a new entity holding object.
	Currently works for strings and byte arrays.
	You will probably want to set the mime type to something more specific"
	
	^ object isString 
		ifTrue: [ self text: object ] 
		ifFalse: [ self bytes: object ]! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/17/2010 20:39'!
with: object type: type
	"Create a new entity holding object.
	Currently works for strings and byte arrays.
	The entity is set to mime type"
	
	^ (self with: object)
		contentType: type;
		yourself! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:17'!
contentLength 
	"Content length can be nil when it is (currently) unknown."
	
	^ contentLength! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:57'!
contentLength: integer
	contentLength := integer! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:57'!
contentType
	^ contentType ! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2010 15:43'!
contentType: object
	"We only allow assignment compatible with our designated mime type"
	| newType |
	newType := ZnUtils asMimeType: object.
	contentType = newType 
		ifTrue: [ ^ self ]
		ifFalse: [ 
			(self class designatedMimeType isNil or: [ self class designatedMimeType = newType ])
				ifTrue: [ contentType := newType ] ]! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2010 22:04'!
contents
	self subclassResponsibility! !

!ZnEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/12/2010 10:03'!
initialize ! !

!ZnEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:23'!
printContentsOn: stream
	stream space.
	self contentType notNil ifTrue: [ 
		stream print: self contentType; space ].
	self contentLength notNil ifTrue: [
		stream print: self contentLength; nextPut: $B; space ]! !

!ZnEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:24'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	self printContentsOn: stream.
	stream nextPut: $)! !

!ZnEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/6/2010 22:02'!
readFrom: stream
	self subclassResponsibility ! !

!ZnEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:30'!
writeOn: stream
	self subclassResponsibility ! !

ZnEntity subclass: #ZnMultiPartFormDataEntity
	instanceVariableNames: 'parts representation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnMultiPartFormDataEntity commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:44' prior: 0!
ZnMultiPartFormDataEntity is a concrete HTTP Entity holding multi part form data that encoded according to specific rules.

Acknowledgement: some code borrowed from AJP.

Part of Zinc HTTP Components.!

!ZnMultiPartFormDataEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 20:42'!
designatedMimeType
	^ ZnMimeType multiPartFormData  ! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 20:07'!
addPart: mimePart
	self parts add: mimePart! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/4/2010 20:11'!
computeRepresentation
	representation := ByteArray streamContents: [ :stream | 
		self writeRepresentationOn: (ZnBivalentWriteStream on: stream) ].
	contentLength := representation size! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 21:17'!
contentType: object
	"We only allow assignment compatible with our designated mime type.
	The main/sub must be equal but the parameters must be allowed to be different"
	
	| newType |
	newType := ZnUtils asMimeType: object.
	self class designatedMimeType = newType
		ifTrue: [ contentType := newType ]! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 19:14'!
contents
	^ self parts! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/5/2010 17:13'!
generateBoundary
	^ String streamContents: [ :stream | | letters |
			stream nextPutAll: 'Boundary-'.
			letters := ($A to: $Z).
			8 timesRepeat: [ stream nextPut: letters atRandom ] ]
	! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 19:17'!
getBoundary
	"Return the multipart/form-data MIME part boundary.
	This should really be there, set externally or internally.
	See #initialize and #mimeTypeWithBoundary"
	
	^ (self contentType parameterAt: 'boundary') withoutQuoting ! !

!ZnMultiPartFormDataEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/5/2010 17:17'!
initialize
	super initialize.
	self contentType: self mimeTypeWithBoundary  
	! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/4/2010 19:15'!
invalidateRepresentation
	representation := nil.
	contentLength := nil! !

!ZnMultiPartFormDataEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:43'!
isEmpty
	^ parts isNil or: [ parts isEmpty ]! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/5/2010 17:19'!
mimeTypeWithBoundary
	| mimeType |
	mimeType := self class designatedMimeType copy.
	mimeType parameterAt: 'boundary' put: self generateBoundary.
	^ mimeType! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/5/2010 19:33'!
parse: bytes boundary: boundary
	| next start fullBoundary |
	fullBoundary := #[45 45], boundary.
	start := fullBoundary size + 1.
	next := bytes
		indexOfSubCollection: fullBoundary
		startingAt: start 
		ifAbsent: [ bytes size ].
	[ next < (bytes size - 2) ] whileTrue: [
		self addPart: (ZnMimePart readFrom: (bytes copyFrom: start + 2 to: next - 3) readStream).
		start := next + fullBoundary size.
		next := bytes
			indexOfSubCollection: fullBoundary
			startingAt: start
			ifAbsent: [ bytes size ] ]! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 19:12'!
partNamed: fieldName
	^ self partNamed: fieldName ifNone: [ self error: 'Cannot find part named ', fieldName asString ]! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 19:18'!
partNamed: fieldName ifNone: block
	parts isNil ifTrue: [ block value ].
	^ self parts detect: [ :each | each fieldName = fieldName ] ifNone: block! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 20:07'!
parts
	parts isNil ifTrue: [ parts := OrderedCollection new ].
	^ parts! !

!ZnMultiPartFormDataEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:44'!
printContentsOn: stream
	super printContentsOn: stream.
	self isEmpty ifFalse: [ 
		self parts printElementsOn: stream.
		stream space ]! !

!ZnMultiPartFormDataEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/4/2010 21:02'!
readFrom: stream
	"Switch to streaming implementation later on"
	
	| bytes |
	self contentLength isNil
		ifTrue: [ 
			bytes := stream upToEnd ]
		ifFalse: [
			bytes := ByteArray ofSize: self contentLength.
			stream next: self contentLength into: bytes ].
	self parse: bytes boundary: self getBoundary asByteArray! !

!ZnMultiPartFormDataEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 10/5/2010 20:28'!
writeOn: stream
	"We always go via our representation because we need a bivalent stream"
	
	representation isNil ifTrue: [ self computeRepresentation ].
	stream nextPutAll: representation! !

!ZnMultiPartFormDataEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 10/5/2010 17:24'!
writeRepresentationOn: stream
	| boundary |
	boundary := self getBoundary.
	self parts do: [ :each |
		stream nextPutAll: '--'; nextPutAll: boundary; nextPutAll: String crlf.
		each writeOn: stream.
		stream nextPutAll: String crlf ].
	stream nextPutAll: '--'; nextPutAll: boundary; nextPutAll: '--'; nextPutAll: String crlf
! !

ZnEntity subclass: #ZnStreamingEntity
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnStreamingEntity commentStamp: '<historical>' prior: 0!
ZnStreamingEntity is a concrete HTTP Entity based on a stream.

When a ZnStreamingEntity is instanciated using #readFrom: 
its content type and length are set but no body is actually read.
Instead, stream is set to a ZnLimitedReadStream that clients can/must
use to read from.

When the client is done, either all data must be read or #consumeContent must be called.
A client should normally not close the stream.

A ZnStreamingEntity can also be instanciated with an existing read stream.
During #writeOn: this stream is copied to the output stream.

Part of Zinc HTTP Components.!

!ZnStreamingEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/15/2010 19:23'!
designatedMimeType
	"I have no designated mime type, I can handle all content.
	See #concreteSubclassForType: "
	
	^ nil! !

!ZnStreamingEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/15/2010 17:13'!
readFrom: stream usingType: mimeType andLength: length
	"Create and return a new streaming entity of mimeType and length from stream.
	Clients can/must use the embedded ZnLimitedReadStream to read all content, 
	or call #consumeContent when they are done. They should not close the stream."
	
	^ (self type: mimeType length: length)
		readFrom: stream;
		yourself
! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 17:10'!
consumeContent
	"To be improved later on using some kind of larger buffer"
	
	[ self stream atEnd ] 
		whileFalse: [ self stream next ]! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/28/2010 13:39'!
contents
	"Since this is a non-repeatable entity, 
	#contents will #consumeContent, 
	this can thus only be done once"
	
	^ self stream upToEnd ! !

!ZnStreamingEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/26/2010 19:34'!
readFrom: readStream
	self contentLength isNil
		ifTrue: [ 
			self stream: readStream ]
		ifFalse: [ | limitedStream |
			limitedStream := ZnLimitedReadStream on: readStream limit: self contentLength.
			self contentType isBinary 
				ifTrue: [ limitedStream binary ] 
				ifFalse: [ limitedStream ascii ]. 
			self stream: limitedStream ]! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:14'!
readStream 
	^ self stream! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 17:04'!
stream 
	^ stream! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 17:04'!
stream: readStream
	stream := readStream! !

!ZnStreamingEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/17/2010 13:43'!
writeOn: outputStream
	ZnUtils 
		streamFrom: self stream 
		to: outputStream 
		size: self contentLength.
	stream close! !

ZnEntity subclass: #ZnStringEntity
	instanceVariableNames: 'string encoder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnStringEntity commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:45' prior: 0!
ZnStringEntity is a concrete HTTP Entity based on a String.
It is used to hold textual (non-binary) data.
Optionally, an encoding is used to convert to and from bytes.
The default encoding it UTF-8.

Part of Zinc HTTP Components.!

!ZnStringEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:58'!
designatedMimeType
	"I have no designated mime type, I can handle all non-binary content.
	See #concreteSubclassForType: "
	
	^ nil! !

!ZnStringEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/2/2010 12:22'!
html: string
	^ (self type: ZnMimeType textHtml)
		string: string;
		yourself! !

!ZnStringEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/2/2010 12:22'!
text: string
	^ (self type: ZnMimeType textPlain)
		string: string;
		yourself! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/2/2010 13:31'!
computeContentLength
	| length |
	self string isNil ifTrue: [ ^ self ].
	length := self string 
		inject: 0 
		into: [ :sum :each | sum + (self encoder encodedByteCountFor: each) ].
	super contentLength: length! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/2/2010 12:25'!
contentLength
	super contentLength isNil ifTrue: [ self computeContentLength ].
	^ super contentLength! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:57'!
contents
	^ self string! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/2/2010 13:08'!
encoder
	encoder isNil ifTrue: [ self initializeEncoder ].
	^ encoder! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 15:11'!
encoder: anEncoding
	(encoder notNil and: [ anEncoding ~= encoder ]) ifTrue: [ self invalidateContentLength ].
	encoder := anEncoding! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/2/2010 13:08'!
initializeEncoder
	self contentType isCharSetUTF8
		ifTrue: [ self encoder: ZnUTF8Encoder new ]
		ifFalse: [ self encoder: ZnNullEncoder new ]! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/2/2010 12:20'!
invalidateContentLength
	self contentLength: nil! !

!ZnStringEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/6/2010 15:11'!
printContentsOn: stream
	super printContentsOn: stream.
	self string notNil ifTrue: [ 
		stream nextPutAll: self string; space ]! !

!ZnStringEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/3/2010 16:12'!
readFrom: stream
	| readStream readBlock newString |
	readStream := self contentLength isNil 
		ifTrue: [ stream ] 
		ifFalse: [ ZnLimitedReadStream on: stream limit: self contentLength ].
	self initializeEncoder.
	readBlock := [ :stringStream |
		[ readStream atEnd ] whileFalse: [
			stringStream nextPut: (encoder nextFromStream: readStream) ] ].
	newString := self contentLength isNil
		ifTrue: [
			String streamContents: readBlock ]
		ifFalse: [
			String new: self contentLength streamContents: readBlock ].
	self string: newString! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:57'!
readStream
	^ self string readStream! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:57'!
string
	^ string! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 15:11'!
string: aString
	(string notNil and: [ aString ~= string ]) ifTrue: [ self invalidateContentLength ].
	string := aString! !

!ZnStringEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/2/2010 13:09'!
writeOn: stream
	self initializeEncoder.
	self string do: [ :each | 
		encoder nextPut: each toStream: stream ]! !

Object subclass: #ZnEntityReader
	instanceVariableNames: 'headers stream streaming allowReadingUpToEnd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnEntityReader commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:59' prior: 0!
I am ZnEntityReader, I help reading ZnEntities from a stream given meta data in headers.

I deal with chunking and gzip decoding.

Part of Zinc HTTP Components.!

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/3/2010 16:07'!
allowReadingUpToEnd
	allowReadingUpToEnd := true! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/3/2010 16:07'!
allowsReadingUpToEnd
	^ allowReadingUpToEnd notNil and: [ allowReadingUpToEnd ]! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/3/2010 16:09'!
canReadContent
	^ (self hasContentLength or: [ self isChunked ]) or: [ self allowsReadingUpToEnd ]! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 23:10'!
contentLength
	^ (self headers includesKey: 'Content-Length')
		ifTrue: [ self headers contentLength ]
		ifFalse: [ nil ]! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 23:11'!
contentType
	^ (self headers includesKey: 'Content-Type') 
		ifTrue: [ self headers contentType ] 
		ifFalse: [ ZnMimeType applicationOctetStream ]! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:35'!
hasContentLength
	^ self headers includesKey: 'Content-Length'! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:23'!
headers
	^ headers! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:22'!
headers: znHeaders
	headers := znHeaders! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/26/2010 13:10'!
isChunked
	| transferEncoding |
	transferEncoding := self headers at: 'Transfer-Encoding' ifAbsent: [ ^ false ].
	^ transferEncoding beginsWith: 'chunked'! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/27/2010 13:55'!
isGzipped
	| contentEncoding |
	contentEncoding := self headers at: 'Content-Encoding' ifAbsent: [ ^ false ].
	^ contentEncoding = 'gzip'! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/27/2010 16:45'!
isIdentityEncoded
	^ self isGzipped not! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/25/2010 23:03'!
isStreaming
	^ streaming notNil and: [ streaming ]! !

!ZnEntityReader methodsFor: 'public' stamp: 'SvenVanCaekenberghe 10/3/2010 16:09'!
readEntity
	| entity decodedEntityLength chunkedStream |
	self canReadContent ifFalse: [ ^ nil ].
	chunkedStream := self isChunked ifTrue: [ stream := ZnChunkedReadStream on: stream ] ifFalse: [ nil ].
	self isGzipped 
		ifTrue: [ 
			self hasContentLength ifTrue: [ stream := ZnLimitedReadStream on: stream limit: self contentLength ].
			decodedEntityLength := nil.
			stream := GZipReadStream on: stream ]
		ifFalse: [ decodedEntityLength := self contentLength ].
	entity := (self isStreaming ifTrue: [ ZnStreamingEntity ] ifFalse: [ ZnEntity ])
			readFrom: stream
			usingType: self contentType 
			andLength: decodedEntityLength.
	self isChunked ifTrue: [ 
		self isIdentityEncoded ifTrue: [ entity contentLength: stream totalSize ].
		self headers addAll: chunkedStream extraHeaders ].
	^ entity! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:23'!
stream
	^ stream! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:22'!
stream: readStream
	stream := readStream ! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:31'!
streaming
	streaming := true! !

Object subclass: #ZnFixedClient
	instanceVariableNames: 'host port connection username password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnFixedClient commentStamp: 'SvenVanCaekenberghe 12/7/2010 15:22' prior: 0!
I am ZnFixedClient.
I implement the client part of the HTTP protocol. 

I talk to one fixed host:port combination, reusing the same connection for each request.
If I lose the connection, I simply reconnect. 
If something goes wrong, I do one retry.
I optionally use basic authentication.

(ZnFixedClient host: 'caretaker.wolf359.be')
	get: 'index.html';
	get: 'small.html'

Part of Zinc HTTP Components.!

!ZnFixedClient class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/25/2010 21:04'!
host: hostname
	^ self host: hostname port: 80! !

!ZnFixedClient class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/25/2010 20:06'!
host: hostname port: port
	^ self new
		host: hostname;
		port: port;
		yourself! !

!ZnFixedClient methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/25/2010 22:06'!
close
	connection notNil ifTrue: [ 
		connection close. 
		connection := nil ]! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/28/2010 12:30'!
connection
	self isConnected
		ifFalse: [ connection := self newConnection ].
	^ connection! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/25/2010 22:05'!
delete: uri
	| request |
	request := ZnRequest delete: (self fixedUrl: uri).
	^ self executeRequest: request! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/28/2010 14:17'!
executeRequest: request
	| response |
	self preProcessRequest: request.
	^ (response := self executeRequestOnceIgnoringErrors: request) isNil
		ifTrue: [
			[ self close; executeRequestOnce: request ] ifCurtailed: [ self close ] ]
		ifFalse: [
			response ]! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 23:21'!
executeRequestOnce: request
	| stream response |
	stream := self connection.
	request writeOn: stream.
	stream flush.
	response := ZnResponse readFrom: stream.
	self postProcessResponse: response.
	^ response! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/28/2010 13:42'!
executeRequestOnceIgnoringErrors: request
	^ [ self executeRequestOnce: request ] on: Error do: [ nil ]! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/25/2010 21:00'!
fixedUrl: uri
	| fixedPart |
	fixedPart := String streamContents: [ :stream |
		stream nextPutAll: 'http://'; nextPutAll: self host; nextPut: $:; print: self port ].
	^ fixedPart asUrl newFromRelativeText: uri! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/25/2010 20:41'!
get: uri
	| request |
	request := ZnRequest get: (self fixedUrl: uri).
	^ self executeRequest: request! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:06'!
host
	^ host! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:06'!
host: hostname
	host := hostname! !

!ZnFixedClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/28/2010 13:28'!
isConnected
	^ connection notNil and: [ connection isConnected and: [ connection isOtherEndConnected ] ]! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/28/2010 12:35'!
newConnection
	^ (SocketStream openConnectionToHostNamed: self host port: self port)
			binary;
			yourself! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:11'!
port
	port isNil ifTrue: [ port := 80 ].
	^ port! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:07'!
port: integer
	port := integer! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/25/2010 22:05'!
post: uri data: entity
	| request |
	(request := ZnRequest post: (self fixedUrl: uri))
		entity: entity.
	^ self executeRequest: request! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 23:21'!
postProcessResponse: response
	response isConnectionClose
		ifTrue: [ self close ]
	! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 23:22'!
preProcessRequest: request
	(username notNil and: [ password notNil ]) ifTrue: [
		request setBasicAuthenticationUsername: username password: password ]! !

!ZnFixedClient methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:53'!
printOn: stream
	super printOn: stream.
	self host notNil ifTrue: [
		stream nextPut: $(; nextPutAll: 'http://'; nextPutAll: self host; nextPut: $:; print: self port; nextPut: $) ]! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/25/2010 22:06'!
put: uri data: entity
	| request |
	(request := ZnRequest put: (self fixedUrl: uri))
		entity: entity.
	^ self executeRequest: request! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 21:02'!
username: user password: pwd
	username := user.
	password := pwd! !

Object subclass: #ZnHTTPSocketFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnHTTPSocketFacade commentStamp: '<historical>' prior: 0!
I am ZnHTTPSocketFacade.

On my class side I implement all necessary replacement methods of HTTPSocket.

The interface (signature and semantics) of this API is defined by HTTPSocket.

Overwriting these HTTPSocket methods with mine will effectively redirect all
HTTP client access in this Smalltalk image through Zinc HTTP Components.

Part of Zinc HTTP Components.!

!ZnHTTPSocketFacade class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/21/2010 08:12'!
execute: request on: stream
	"All our requests are one shot"
	
	request setConnectionClose.
	^ [ ZnClient execute: request on: stream ] ensure: [ stream close ]! !

!ZnHTTPSocketFacade class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/13/2010 21:47'!
extendHeaders: headers with: object 
	(object isNil or: [ object isEmpty ]) ifTrue: [ ^ headers ].
	object isString 
		ifTrue: [ headers addAll: (ZnHeaders readFrom: object readStream) ] 
		ifFalse: [ headers withAll: object ].
	^ headers
	
! !

!ZnHTTPSocketFacade class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/13/2010 21:37'!
extendUrl: urlObject withArguments: arguments
	| url query |
	url := urlObject asUrl.
	(arguments isNil or: [ arguments isEmpty ]) ifTrue: [ ^ url ].
	query := arguments isString
		ifTrue: [ arguments first = $? 
					ifTrue: [ arguments ] 
					ifFalse: [ '?', arguments ] ] 
		ifFalse: [ String streamContents: [ :stream |
					stream nextPut: $?.
					ZnUtils writeQueryFields: arguments on: stream ] ].
	^ (url printString, query) asUrl
! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:17'!
httpGet: url
	^ self 
		httpGet: url 
		accept: '*/*'! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:17'!
httpGet: url accept: mimeType
	^ self 
		httpGet: url 
		args: nil 
		accept: '*/*'! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:17'!
httpGet: url args: queryArguments accept: mimeType
	^ self 
		httpGet: url 
		args: queryArguments 
		accept: mimeType 
		request: nil! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 13:22'!
httpGet: urlObject args: queryArguments accept: mimeType request: extraHeaders
	| url stream request response |
	url := self extendUrl: urlObject withArguments: queryArguments. 
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest get: url) 
		setAccept: mimeType.
	self extendHeaders: request headers with: extraHeaders. 
	response := self execute: request on: stream.
	^ self streamOrErrorStringFrom: response uri: url
! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 13:22'!
httpGet: urlObject args: queryArguments user: username passwd: password
	| url stream request response |
	url := self extendUrl: urlObject withArguments: queryArguments. 
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest get: url) 
		setBasicAuthenticationUsername: username password: password.
	response := self execute: request on: stream.
	^ self streamOrErrorStringFrom: response uri: url! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:17'!
httpGetDocument: url
	^ self 
		httpGetDocument: url 
		accept: 'application/octet-stream'! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:17'!
httpGetDocument: url accept: mimeType
	^ self 
		httpGetDocument: url 
		args: nil 
		accept: 'application/octet-stream'! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:18'!
httpGetDocument: url args: queryArguments
	^ self 
		httpGetDocument: url 
		args: queryArguments 
		accept: 'application/octet-stream'! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:18'!
httpGetDocument: url args: queryArguments accept: mimeType
	^ self 
		httpGetDocument: url 
		args: queryArguments 
		accept: 'application/octet-stream' 
		request: nil! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 13:22'!
httpGetDocument: urlObject args: queryArguments accept: mimeType request: extraHeaders
	| url stream request response |
	url := self extendUrl: urlObject withArguments: queryArguments. 
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest get: url) 
		setAccept: mimeType.
	self extendHeaders: request headers with: extraHeaders. 
	response := self execute: request on: stream.
	^ self mimeDocumentOrErrorStringFrom: response uri: url
! !

!ZnHTTPSocketFacade class methodsFor: 'operations images' stamp: 'SvenVanCaekenberghe 9/13/2010 16:07'!
httpGif: url
	^ ZnClient getGif: url! !

!ZnHTTPSocketFacade class methodsFor: 'operations images' stamp: 'SvenVanCaekenberghe 9/13/2010 16:08'!
httpJpeg: url
	^ ZnClient getJpeg: url! !

!ZnHTTPSocketFacade class methodsFor: 'operations images' stamp: 'SvenVanCaekenberghe 9/13/2010 16:08'!
httpPng: url
	^ ZnClient getPng: url! !

!ZnHTTPSocketFacade class methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 9/14/2010 14:34'!
httpPost: urlObject args: arguments accept: mimeType
	| url stream request response |
	url := self extendUrl: urlObject withArguments: nil. 
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest post: url) 
		setAccept: mimeType;
		entity: (ZnApplicationFormUrlEncodedEntity withAll: arguments).
	response := self execute: request on: stream.
	^ self streamOrErrorStringFrom: response uri: url
! !

!ZnHTTPSocketFacade class methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 9/14/2010 14:34'!
httpPost: urlObject args: arguments user: username passwd: password
	| url stream request response |
	url := self extendUrl: urlObject withArguments: nil. 
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest post: url) 
		setBasicAuthenticationUsername: username password: password;
		entity: (ZnApplicationFormUrlEncodedEntity withAll: arguments).
	response := self execute: request on: stream.
	^ self mimeDocumentOrErrorStringFrom: response uri: url

! !

!ZnHTTPSocketFacade class methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 9/17/2010 20:05'!
httpPostDocument: url args: arguments
	^ self 
		httpPostDocument: url 
		args: arguments 
		accept: 'application/octet-stream'! !

!ZnHTTPSocketFacade class methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 9/17/2010 20:05'!
httpPostDocument: url args: arguments accept: mimeType
	^ self 
		httpPostDocument: url 
		args: arguments 
		accept: mimeType 
		request: nil! !

!ZnHTTPSocketFacade class methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 9/17/2010 20:37'!
httpPostDocument: urlObject args: arguments accept: mimeType request: extraHeaders
	| url stream request response |
	url := self extendUrl: urlObject withArguments: nil. 
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest post: url) 
		setAccept: mimeType;
		entity: (ZnApplicationFormUrlEncodedEntity withAll: arguments).
	self extendHeaders: request headers with: extraHeaders. 
	response := self execute: request on: stream.
	^ self mimeDocumentOrErrorStringFrom: response uri: url
! !

!ZnHTTPSocketFacade class methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 9/13/2010 21:04'!
httpPostMultipart: url args: arguments accept: mimeType request: extraHeaders
	self shouldBeImplemented ! !

!ZnHTTPSocketFacade class methodsFor: 'operations put' stamp: 'SvenVanCaekenberghe 12/4/2010 14:21'!
httpPut: string to: urlObject user: username passwd: password 
	"We're implementing the weird semantics of the original ;-)"

	| url stream request response |
	url := self extendUrl: urlObject withArguments: nil. 
	stream := ZnUtils socketStreamToUrl: url.
	(request := ZnRequest put: url) 
		setBasicAuthenticationUsername: username password: password; 
		entity: (ZnEntity with: string).
	request headers removeKey: 'Content-Type'.
	response := self execute: request on: stream.
	^ String streamContents: [ :stringStream | response writeOn: stringStream ]! !

!ZnHTTPSocketFacade class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/21/2010 16:14'!
mimeDocumentOrErrorStringFrom: response uri: uri
	^ (#(200 201) includes: response statusLine code)
		ifTrue: [
			MIMEDocument
				contentType: response contentType asString
				content: response entity contents 
				url: uri ]
		ifFalse: [ 
			String streamContents: [ :stream |
				stream nextPutAll: 'HTTP request for '; print: uri; nextPutAll: ' failed with '.
				response statusLine writeOn: stream ] ]
		! !

!ZnHTTPSocketFacade class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2010 20:42'!
streamOrErrorStringFrom: response uri: uri
	^ (#(200 201) includes: response statusLine code)
		ifTrue: [
			(RWBinaryOrTextStream with: response entity contents) 
				reset; 
				yourself ]
		ifFalse: [ 
			String streamContents: [ :stream |
				stream nextPutAll: 'HTTP request for '; print: uri; nextPutAll: ' failed with '.
				response statusLine writeOn: stream ] ]
		! !

Object subclass: #ZnHeaders
	instanceVariableNames: 'headers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnHeaders commentStamp: '<historical>' prior: 0!
I am ZnHeaders.
I am a collection of HTTP headers.
I can be used for generating and parsing.

Headers names are normalized and used case insensitive.
Header values can be multi-valued.

Part of Zinc HTTP Components.!

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 21:12'!
defaultRequestHeaders
	^ self new
		at: 'User-Agent' put: ZnConstants defaultUserAgent;
		at: 'Accept' put: '*/*';
		yourself! !

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/8/2010 09:51'!
defaultResponseHeaders
	^ self new
		at: 'Server' put: ZnConstants serverString;
		at: 'Date' put: ZnUtils httpDate;
		yourself! !

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 17:07'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 21:21'!
requestHeadersFor: url
	^ self defaultRequestHeaders
		at: 'Host' put: (ZnUtils hostAndPortOf: url);
		yourself! !

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/13/2010 21:45'!
withAll: keyedCollection
	^ self new
		addAll: keyedCollection;
		yourself ! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 15:05'!
acceptEntityDescription: entity
	"Take over the content type and length from entity"
	
	self
		at: 'Content-Type' put: entity contentType printString;
		at: 'Content-Length' put: entity contentLength printString! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2010 21:44'!
addAll: keyedCollection
	keyedCollection keysAndValuesDo: [ :key :value |
		self at: key put: value ].
	^ keyedCollection ! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:25'!
at: headerName
	"Return the value stored under headerName,
	a String for single-valued headers, an Array of Strings for multi-valued headers"
	
	^ self headers at: (self normalizeHeaderKey: headerName)! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:48'!
at: headerName add: value
	"Store value under headerName, optionally turning it into a multi-valued header 
	when a value was already present"
	
	| normalizedKey existingValue newValue |
	normalizedKey := self normalizeHeaderKey: headerName.
	existingValue := self headers at: normalizedKey ifAbsent: [ nil ].
	newValue := existingValue isNil
		ifTrue: [ value ] 
		ifFalse: [
			existingValue isArray 
				ifTrue: [ existingValue copyWith: value ]
				ifFalse: [ Array with: existingValue with: value ] ].
	^ self headers at: normalizedKey put: newValue 
		
! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:16'!
at: headerName ifAbsent: block
	"Like #at: but executes block when the headerName is not found"
	
	self isEmpty ifTrue: [ ^ block value ].
	^ self headers at: (self normalizeHeaderKey: headerName) ifAbsent: block! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:26'!
at: headerName put: value
	"Store value under headerName, replace existing entries"
	
	^ self headers at: (self normalizeHeaderKey: headerName) put: value! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:49'!
at: headerName put: value ifPresentMerge: binaryBlock
	"Store value under headerName, when there is an existing entry,
	stored the result of evaluating binaryBlock with old and new value"
	
	| normalizedKey existingValue newValue |
	normalizedKey := self normalizeHeaderKey: headerName.
	existingValue := self headers at: normalizedKey ifAbsent: [ nil ].
	newValue := existingValue isNil
		ifTrue: [ value ] 
		ifFalse: [ binaryBlock value: existingValue value: value ].
	^ self headers at: normalizedKey put: newValue 
		

! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:27'!
contentLength
	^ Integer 
		readFrom: (self headers at: 'Content-Length') 
		ifFail: [ self error: 'Illegal HTTP Content Length' ]! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:27'!
contentType
	^ ZnMimeType fromString: (self headers at: 'Content-Type')! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:12'!
hasContentLength
	^ self includesKey: 'Content-Length'! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:11'!
hasContentType
	^ self includesKey: 'Content-Type'! !

!ZnHeaders methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/6/2010 17:03'!
headers
	headers isNil ifTrue: [ headers := Dictionary new ].
	^ headers! !

!ZnHeaders methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/21/2010 12:14'!
headersDo: block
	"Execute a two argument (key, value) block for each header.
	Multi-valued headers are handled transparently."

	self isEmpty ifTrue: [ ^ self ].
	self headers keysAndValuesDo: [ :headerKey :headerValue |
		headerValue isArray
			ifTrue: [ 
				headerValue do: [ :each | 
					block value: headerKey value: each ] ]
			ifFalse: [
				block value: headerKey value: headerValue ] ]! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:13'!
includesKey: headerName
	^ self isEmpty not and: [ self headers includesKey: (self normalizeHeaderKey: headerName) ]
	! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:29'!
isDescribingEntity
	"Do I include enough information to describe an entity (i.e. content length and type) ?"
	
	^ (self headers includesKey: 'Content-Type') 
		and: [ self headers includesKey: 'Content-Length' ]
! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/6/2010 17:09'!
isEmpty 
	^ headers isNil or: [ self headers isEmpty ]! !

!ZnHeaders methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/26/2010 18:13'!
keysAndValuesDo: block
	"Execute a two argument (key, value) block for each header.
	Multi-valued headers are handled transparently."

	self headersDo: block! !

!ZnHeaders methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/10/2010 20:04'!
normalizeHeaderKey: string
	"Test string to see if it has proper header key capitalization,
	if true, return string, if not, return a properly capitalized copy"
	
	^ (ZnUtils isCapitalizedString: string)
		ifTrue: [ string ] 
		ifFalse: [ ZnUtils capitalizeString: string ] ! !

!ZnHeaders methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/20/2010 17:20'!
printOn: stream
	super printOn: stream.
	self isEmpty ifFalse: [ self headers printElementsOn: stream ]
	! !

!ZnHeaders methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 19:26'!
readFrom: stream
	| line reader |
	reader := ZnLineReader on: stream.
	[ (line := reader nextLine) isEmpty ] whileFalse: [
		self readOneHeaderFrom: line readStream ] ! !

!ZnHeaders methodsFor: 'private' stamp: 'MattKennedy 9/28/2010 13:14'!
readOneHeaderFrom: stream
	| key value |
	key := stream upTo: $:.
	[ stream peek == Character space ] whileTrue: [ stream next ].
	value := stream upToEnd.
	self at: key add: value! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 13:36'!
removeKey: headerName	
	^ self headers removeKey: (self normalizeHeaderKey: headerName)! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:16'!
removeKey: headerName ifAbsent: block
	self isEmpty ifTrue: [ ^ block value ].
	^ self headers removeKey: (self normalizeHeaderKey: headerName) ifAbsent: block! !

!ZnHeaders methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:39'!
writeOn: stream
	| writeStream |
	writeStream := ZnBivalentWriteStream on: stream.
	self headersDo: [ :key :value |
		writeStream nextPutAll: key; nextPut: $:; space; nextPutAll: value; nextPutAll: String crlf ]! !

Object subclass: #ZnLineReader
	instanceVariableNames: 'stream buffer position limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnLineReader commentStamp: 'SvenVanCaekenberghe 9/27/2010 17:31' prior: 0!
I am ZnLineReader and I help reading CRLF terminated lines from a stream.
I can work on ascii and binary streams doing conversions as necessary.
I limit lines to a maximum length.
I work with a reusable buffer.

Part of Zinc HTTP Components.!

!ZnLineReader class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/27/2010 17:15'!
on: stream
	^ self new
		on: stream;
		yourself! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 19:34'!
growBuffer
	| newBuffer |
	buffer size >= limit ifTrue: [ self error: 'Maximum line length reached' ].
	newBuffer := String new: buffer size * 2.
	newBuffer replaceFrom: 1 to: buffer size with: buffer.
	buffer := newBuffer ! !

!ZnLineReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 17:30'!
limit: size
	limit := size! !

!ZnLineReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 19:05'!
nextLine
	self reset.
	[ self processNext isNil ] whileFalse.
	^ buffer copyFrom: 1 to: position ! !

!ZnLineReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 17:30'!
on: readStream
	stream := readStream.
	buffer := String new: 64.
	self limit: 4096; reset! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 19:04'!
processNext
	| item |
	stream atEnd ifTrue: [ ^ nil ].
	item := stream next.
	stream isBinary ifTrue: [ item := item asCharacter ].
	item == Character lf
		ifTrue: [
			(position > 0 and: [ (buffer at: position) == Character cr ])
				ifTrue: [
					position := position - 1.
					^ nil ] ].
	self store: item.
	^ item
			
	! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 17:29'!
reset
	position := 0! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 18:43'!
store: item
	position >= buffer size ifTrue: [ self growBuffer ].
	position := position + 1.
	buffer at: position put: item		
	! !

Object subclass: #ZnMessage
	instanceVariableNames: 'headers entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnMessage commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:46' prior: 0!
I am ZnMessage, an abstract class representing an HTTP Message.

Part of Zinc HTTP Components.!

!ZnMessage class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:56'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnMessage class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 12:18'!
readHeaderFrom: stream
	^ self new
		readHeaderFrom: stream;
		yourself! !

!ZnMessage class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:56'!
readStreamingFrom: stream
	^ self new
		readStreamingFrom: stream;
		yourself! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:17'!
contentLength
	self hasEntity ifTrue: [ ^ self entity contentLength ].
	(self hasHeaders and: [ self headers hasContentLength ]) ifTrue: [ ^ self headers contentLength ].
	^ nil! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:15'!
contentType
	self hasEntity ifTrue: [ ^ self entity contentType ].
	(self hasHeaders and: [ self headers hasContentType ]) ifTrue: [ ^ self headers contentType ].
	^ nil! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/28/2010 13:03'!
contents
	^ entity isNil ifTrue: [ nil ] ifFalse: [ entity contents ]! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:34'!
entity
	^ entity! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:56'!
entity: object
	(entity := object) isNil ifTrue: [ ^ self ].
	self headers isDescribingEntity ifFalse: [
		self headers acceptEntityDescription: object ]! !

!ZnMessage methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/25/2010 22:25'!
entityReaderOn: stream
	^ ZnEntityReader new
		headers: self headers;
		stream: stream;
		yourself! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:02'!
hasEntity
	^ self entity notNil! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 20:32'!
hasHeaders
	^ headers notNil and: [ self headers isEmpty not ]! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:34'!
headers
	headers isNil ifTrue: [ headers := ZnHeaders new ].
	^ headers! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headers: object
	headers := object! !

!ZnMessage methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/21/2010 12:04'!
headersDo: twoArgumentBlock
	self hasHeaders
		ifTrue: [ self headers headersDo: twoArgumentBlock ]! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/27/2010 20:14'!
isConnectionClose
	^ (self headers at: 'Connection' ifAbsent: [ nil ]) = 'close'! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/25/2010 22:43'!
readFrom: stream
	self readHeaderFrom: stream.
	self entity: (self entityReaderOn: stream) readEntity
	! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/21/2010 11:32'!
readHeaderFrom: stream
	self headers: (ZnHeaders readFrom: stream)
	! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 12:17'!
readStreamingFrom: stream
	| entityReader |
	self readHeaderFrom: stream.
	(entityReader := self entityReaderOn: stream)
		streaming.
	self entity: entityReader readEntity
	! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 22:32'!
setConnectionClose
	self headers at: 'Connection' put: 'close'! !

!ZnMessage methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/3/2010 21:34'!
writeOn: stream
	| bivalentWriteStream |
	self headers writeOn: stream.
	bivalentWriteStream := ZnBivalentWriteStream on: stream.
	bivalentWriteStream nextPutAll: String crlf.
	self hasEntity ifTrue: [ self entity writeOn: bivalentWriteStream ]! !

ZnMessage subclass: #ZnRequest
	instanceVariableNames: 'requestLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnRequest commentStamp: '<historical>' prior: 0!
ZnRequest represents an HTTP Request, consisting of a request line, headers and an optional entity (body).
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:07'!
delete: urlObject
	^ self method: #DELETE url: urlObject! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:00'!
get: urlObject
	^ self method: #GET url: urlObject! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:07'!
head: urlObject
	^ self method: #HEAD url: urlObject! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:00'!
method: method url: urlObject
	| url |
	url := urlObject asUrl.
	^ self new
		requestLine: (ZnRequestLine method: method uri: url);
		headers: (ZnHeaders requestHeadersFor: url);
		yourself! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:08'!
post: urlObject
	^ self method: #POST url: urlObject! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:08'!
put: urlObject
	^ self method: #PUT url: urlObject! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/8/2010 10:35'!
method
	^ self requestLine method ! !

!ZnRequest methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/21/2010 11:33'!
readHeaderFrom: stream
	self requestLine: (ZnRequestLine readFrom: stream).
	super readHeaderFrom: stream
	! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:06'!
requestLine
	^ requestLine ! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:06'!
requestLine: object
	requestLine := object! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 20:00'!
setAccept: object
	self headers at: 'Accept' put: object asString! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 19:28'!
setAcceptEncodingGzip
	self headers at: 'Accept-Encoding' put: 'gzip'! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/12/2010 20:27'!
setBasicAuthenticationUsername: username password: password
	self headers 
		at: 'Authorization' 
		put: 'Basic ', (username, ':', password) base64Encoded 
! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/8/2010 10:36'!
uri
	^ self requestLine uri ! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 21:48'!
url
	^ self uri ! !

!ZnRequest methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/15/2010 14:10'!
writeOn: stream
	self requestLine writeOn: stream.
	super writeOn: stream! !

ZnMessage subclass: #ZnResponse
	instanceVariableNames: 'statusLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnResponse commentStamp: '<historical>' prior: 0!
ZnResponse represents an HTTP Response consisting of a status line, headers and an optional entity (body).
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:03'!
badRequest: request
	^ self new
		statusLine: (ZnStatusLine code: 400);
		headers: ZnHeaders defaultResponseHeaders;
		entity: (ZnEntity text: 'Bad Request ', request method, ' ', request uri printString, String crlf);
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:03'!
created: uri
	| location |
	location := ZnUtils urlPathQueryFragmentOf: uri.
	^ self new
		statusLine: (ZnStatusLine code: 201);
		headers: ZnHeaders defaultResponseHeaders;
		setLocation: location;
		entity: (ZnEntity text: 'Created ', location, String crlf);
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:03'!
notFound: uri
	^ self new
		statusLine: (ZnStatusLine code: 404);
		headers: ZnHeaders defaultResponseHeaders;
		entity: (ZnEntity text: 'Not Found ', (ZnUtils urlPathQueryFragmentOf: uri), String crlf);
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/17/2010 16:38'!
ok: entity
	^ self new
		statusLine: (ZnStatusLine code: 200);
		headers: ZnHeaders defaultResponseHeaders;
		entity: entity;
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:03'!
redirect: url
	| location |
	location := ZnUtils urlPathQueryFragmentOf: url.
	^ self new
		statusLine: (ZnStatusLine code: 302);
		headers: ZnHeaders defaultResponseHeaders;
		setLocation: location;
		entity: (ZnEntity text: 'Redirect ', location, String crlf);
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'MattKennedy 9/30/2010 15:46'!
unauthorized
	^ self unauthorized: 'Basic realm=ZincHTTPComponents'! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:03'!
unauthorized: authString
	^ self new
		statusLine: (ZnStatusLine code: 401);
		headers: ZnHeaders defaultResponseHeaders;
		setWWWAuthenticate: authString;
		entity: (ZnEntity text: 'Unauthorized', String crlf);
		yourself! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/16/2010 20:44'!
code
	^ self statusLine code! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/28/2010 12:38'!
isSuccess
	^ #(200 201 202 204) includes: self code! !

!ZnResponse methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/21/2010 11:34'!
readHeaderFrom: stream
	self statusLine: (ZnStatusLine readFrom: stream).
	super readHeaderFrom: stream
	! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 15:19'!
setLocation: uriString
	self headers at: 'Location' put: uriString! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 15:19'!
setWWWAuthenticate: string
	self headers at: 'WWW-Authenticate' put: string! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 18:35'!
status
	^ self statusLine code! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:36'!
statusLine
	^ statusLine! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:36'!
statusLine: object
	statusLine := object! !

!ZnResponse methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/15/2010 14:10'!
writeOn: stream
	self statusLine writeOn: stream.
	super writeOn: stream! !

Object subclass: #ZnMimePart
	instanceVariableNames: 'headers entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnMimePart commentStamp: 'SvenVanCaekenberghe 10/5/2010 19:20' prior: 0!
I am ZnMimePart.
I hold headers and an optional entity.
I am used in MultiPartMimeEncoding.

Acknowledgement: some code borrowed from AJP.

Part of Zinc HTTP Components.!

!ZnMimePart class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 10/4/2010 20:57'!
fieldName: fieldName fileName: fileName entity: entity
	^ self new
		setContentDisposition: 'formdata;name="', fieldName, '";filename="', fileName, '"';
		entity: entity;
		yourself! !

!ZnMimePart class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:04'!
fieldName: fieldName value: fieldValue
	^ self new
		setContentDisposition: 'formdata;name="', fieldName, '"';
		entity: (ZnEntity bytes: fieldValue asByteArray);
		yourself! !

!ZnMimePart class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 19:57'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/3/2010 16:12'!
contentDisposition
	^ self headers at: 'Content-Disposition' ifAbsent: [ nil ]! !

!ZnMimePart methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/6/2010 09:30'!
contentDispositionValues
	| contentDisposition values start semiColonIndex |
	(contentDisposition := self contentDisposition) isNil ifTrue: [ ^ #() ].
	values := OrderedCollection new: 3.
	start := 1.
	semiColonIndex := contentDisposition indexOf: $; startingAt: start ifAbsent: [ 0 ].
	[ semiColonIndex isZero ] whileFalse: [
		| value |
		value := contentDisposition copyFrom: start to: semiColonIndex - 1.
		values add: (ZnUtils trimString: value).
		start := semiColonIndex + 1.
		semiColonIndex := contentDisposition indexOf: $; startingAt: start ifAbsent: [ 0 ] ].
	values add: (ZnUtils trimString: (contentDisposition copyFrom: start to: contentDisposition size)).
	^ values! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
contentLength
	self hasEntity ifTrue: [ ^ self entity contentLength ].
	(self hasHeaders and: [ self headers hasContentLength ]) ifTrue: [ ^ self headers contentLength ].
	^ nil! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
contentType
	self hasEntity ifTrue: [ ^ self entity contentType ].
	(self hasHeaders and: [ self headers hasContentType ]) ifTrue: [ ^ self headers contentType ].
	^ nil! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
contents
	^ entity isNil ifTrue: [ nil ] ifFalse: [ entity contents ]! !

!ZnMimePart methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/5/2010 17:01'!
detectContentDispositionValue: aString
	self contentDispositionValues do: [ :each | 
		((each beginsWith: aString)
			and: [ each size > (aString size + 1)
			and: [ (each at: aString size + 1) = $= ] ]) ifTrue: [
				^ (each copyAfter: $=) withoutQuoting ] ].
	^ nil! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
entity
	^ entity! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
entity: object
	(entity := object) isNil ifTrue: [ ^ self ].
	self headers isDescribingEntity ifFalse: [
		self headers acceptEntityDescription: object ]! !

!ZnMimePart methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/3/2010 16:10'!
entityReaderOn: stream
	^ ZnEntityReader new
		headers: self headers;
		stream: stream;
		allowReadingUpToEnd;
		yourself! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 17:03'!
fieldName
	^ self detectContentDispositionValue: 'name'! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 18:54'!
fieldValue
	^ self hasEntity 
		ifTrue: [ self entity contents ] 
		ifFalse: [ nil ] ! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 18:54'!
fieldValueString
	^ self fieldValue asString! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 17:01'!
fileName
	^ self detectContentDispositionValue: 'filename'! !

!ZnMimePart methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
hasEntity
	^ self entity notNil! !

!ZnMimePart methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 13:32'!
hasHeaders
	^ headers notNil and: [ self headers isEmpty not ]! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headers
	headers isNil ifTrue: [ headers := ZnHeaders new ].
	^ headers! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headers: object
	headers := object! !

!ZnMimePart methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headersDo: twoArgumentBlock
	self hasHeaders
		ifTrue: [ self headers headersDo: twoArgumentBlock ]! !

!ZnMimePart methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/30/2010 19:54'!
readFrom: stream
	self headers: (ZnHeaders readFrom: stream).
	self entity: (self entityReaderOn: stream) readEntity
	! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 18:49'!
setContentDisposition: value
	^ self headers at: 'Content-Disposition' put: value ! !

!ZnMimePart methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:54'!
writeOn: stream
	self headers writeOn: stream.
	stream nextPutAll: String crlf.
	self hasEntity ifTrue: [ self entity writeOn: stream ]! !

Object subclass: #ZnMimeType
	instanceVariableNames: 'main sub parameters'
	classVariableNames: 'ExtensionsMap MimeTypes'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnMimeType commentStamp: 'SvenVanCaekenberghe 12/6/2010 13:04' prior: 0!
I am ZnMimeType, a copy from WAMineType.

I know whether I am binary or not and know my charset when applicable.

My class side offers access to some common constants.
Note that for text types we default to UTF-8 encoding.

Part of Zinc HTTP Components.!

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:34'!
applicationFormUrlEncoded
	^ MimeTypes at: #applicationFormUrlEncoded! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:34'!
applicationJavascript
	^ MimeTypes at: #applicationJavascript! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/25/2010 20:43'!
applicationJson
	^ MimeTypes at: #applicationJson! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:34'!
applicationOctetStream
	^ MimeTypes at: #applicationOctetStream! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 15:48'!
default
	^ self applicationOctetStream ! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/16/2010 20:29'!
forFilenameExtension: extension
	^ ExtensionsMap 
		at: extension 
		ifAbsent: [ self default ]! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/14/2010 21:47'!
fromString: aString 
	| main endOfSub sub parts parameters |
	main := aString copyUpTo: $/.
	endOfSub := aString indexOf: $;.
	endOfSub := endOfSub = 0 
		ifTrue: [ aString size ]
		ifFalse: [ endOfSub - 1 ].
	sub := aString copyFrom: main size + 2 to: endOfSub.
	endOfSub = aString size ifTrue: [ ^ self main: main sub: sub ].
	parts := (aString copyFrom: endOfSub + 1 to: aString size) subStrings: ';'.
	parameters := Dictionary new.
	parts do: [ :each | 
		parameters
			at: (each copyUpTo: $=) withBlanksTrimmed 
			put: (each copyAfter: $=) withBlanksTrimmed ].
	^ self 
		main: main
		sub: sub
		parameters: parameters! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:34'!
imageGif
	^ MimeTypes at: #imageGif! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:35'!
imageJpeg
	^ MimeTypes at: #imageJpeg! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:35'!
imagePng
	^ MimeTypes at: #imagePng! !

!ZnMimeType class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 9/16/2010 20:27'!
initialize
	self initializeMimeTypes.
	self initializeExtensionsMap! !

!ZnMimeType class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/16/2010 20:33'!
initializeExtensionsMap 
	"To be extended later"
	
	ExtensionsMap := Dictionary new
		at: 'txt' put: self textPlain;
		at: 'text' put: self textPlain;
		at: 'htm' put: self textHtml;
		at: 'html' put: self textHtml;
		at: 'gif' put: self imageGif;
		at: 'jpg' put: self imageJpeg;
		at: 'jpeg' put: self imageJpeg;
		at: 'png' put: self imagePng;
		at: 'js' put: self applicationJavascript;
		at: 'css' put: self textCss;
		yourself! !

!ZnMimeType class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/2/2010 11:23'!
initializeMimeTypes 
	"To be extended later"

	MimeTypes := Dictionary new
		at: #applicationOctetStream put: (self main: 'application' sub: 'octet-stream');
		at: #applicationJavascript put: (self main: 'application' sub: 'javascript');
		at: #applicationJson put: (self main: 'application' sub: 'json');
		at: #applicationFormUrlEncoded put: (self main: 'application' sub: 'x-www-form-urlencoded');
		at: #imageGif put: (self main: 'image' sub: 'gif');
		at: #imageJpeg put: (self main: 'image' sub: 'jpeg');
		at: #imagePng put: (self main: 'image' sub: 'png');
		at: #textPlain put: ((self main: 'text' sub: 'plain') setCharSetUTF8; yourself);
		at: #textHtml put: ((self main: 'text' sub: 'html') setCharSetUTF8; yourself);
		at: #textCss put: (self main: 'text' sub: 'css');
		at: #multiPartFormData put: (self main: 'multipart' sub: 'form-data');
		yourself! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/7/2010 14:46'!
main: aMainString sub: aSubString
	^self new
		main: aMainString;
		sub: aSubString;
		yourself! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/7/2010 14:46'!
main: aMainString sub: aSubString parameters: aDictionary
	^(self main: aMainString sub: aSubString)
		parameters: aDictionary;
		yourself! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:35'!
multiPartFormData
	^ MimeTypes at: #multiPartFormData! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:35'!
textCss
	^ MimeTypes at: #textCss! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:35'!
textHtml
	^ MimeTypes at: #textHtml! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 20:35'!
textPlain
	^ MimeTypes at: #textPlain! !

!ZnMimeType methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:51'!
= other
	^(other isKindOf: self class)
		and: [ self main = other main
		and: [ self sub = other sub ] ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 10/5/2010 17:23'!
charSet
	^self parameterAt: 'charset' ifAbsent: [ nil ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 12/2/2010 11:24'!
charSet: aString
	self parameters at: 'charset' put: aString! !

!ZnMimeType methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:48'!
hash
	^self main hash bitXor: self sub hash! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:45'!
isBinary
	"answers whether the contents of a document of the receiving mime type are binary"
	| subTypes |
	self main = 'text' ifTrue: [ ^false ].
	self = self class applicationJson ifTrue: [ ^ false ].
	self main = 'application' ifFalse: [ ^true ].
	subTypes := self sub subStrings: '+'.
	^ subTypes noneSatisfy: [ :each | 
		#('x-javascript' 'xml') includes: each ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/2/2010 11:19'!
isCharSetUTF8
	^ self charSet notNil and: [ self charSet sameAs: 'utf-8' ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/17/2010 20:10'!
isNonStandard
	"tests if the receiver is a non-standard mime type that is not registered with IANA"
	^(self main beginsWith: 'x-')
		or: [ (self main beginsWith: 'X-')
		or: [ (self sub beginsWith: 'x-')
		or: [ self sub beginsWith: 'X-' ] ] ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/17/2010 20:10'!
isVendorSpecific
	"tests if the receiver is a vendor specific mimetype"
	^self sub beginsWith: 'vnd.'! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
main
	^ main! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
main: aString
	main := aString! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:51'!
matches: aMimeType
	"aMimeType is the pattern to match, it is a normal instance where main or sub can be wildcards"
	^(aMimeType main = '*'
		or: [ aMimeType main = self main ])
		and: [
			aMimeType sub = '*'
				or: [ aMimeType sub = self sub ] ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 10/5/2010 17:23'!
parameterAt: key
	^ self parameterAt: key ifAbsent: [ self error: 'Parameter not found: ', key asString ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 10/5/2010 19:30'!
parameterAt: key ifAbsent: block
	parameters isNil ifTrue: [ block value ].
	^ self parameters at: key ifAbsent: block! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 9/7/2010 14:50'!
parameterAt: aKey put: aValue
	self parameters at: aKey put: aValue! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:52'!
parameters
	parameters isNil ifTrue: [ parameters := Dictionary new ].
	^ parameters! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
parameters: aDictionary
	parameters := aDictionary! !

!ZnMimeType methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/11/2010 20:49'!
printOn: stream
	stream
		nextPutAll: self main;
		nextPut: $/;
		nextPutAll: self sub.
	parameters isNil ifFalse: [ 
		parameters keysAndValuesDo: [ :key :value | 
			stream
				nextPut: $;;
				nextPutAll: key;
				nextPut: $=;
				nextPutAll: value ] ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 12/2/2010 11:20'!
setCharSetUTF8
	self charSet: 'utf-8'! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
sub
	^ sub! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
sub: aString
	sub := aString! !

Object subclass: #ZnMonticelloServerDelegate
	instanceVariableNames: 'directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnMonticelloServerDelegate commentStamp: '<historical>' prior: 0!
I am ZnMonticelloServerDelegate.
I am a proof of concept implementation of a basic Monticello server.
I function as a delegate for ZnServer.

ZnServer startDefaultOn: 1701.
ZnServer default delegate: ((ZnMonticelloServerDelegate new) 
									directory: (FileDirectory on: '/Users/sven/Tmp/monticello'); 
									yourself).

Part of Zinc HTTP Components.!

!ZnMonticelloServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 10:20'!
directory
	^ directory ! !

!ZnMonticelloServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 10:20'!
directory: fileDirectory
	directory := fileDirectory! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2010 11:04'!
handleGetMczEntry: request
	| name |
	name := request uri path first.
	^ ((self isValidMczName: name) and: [ self directory fileExists: name ]) 
		ifTrue: [ | file entity |
			(file := self directory readOnlyFileNamed: name)
				binary.
			(entity := ZnStreamingEntity type: ZnMimeType applicationOctetStream length: file size)
				stream: file.
			ZnResponse ok: entity ]
		ifFalse: [ ZnResponse notFound: request uri ]! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/3/2010 14:03'!
handleListMczEntries: request
	^ ZnResponse ok: (ZnEntity html: self repositoryListing )! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2010 14:32'!
handlePutMczEntry: request
	| name |
	name := request uri path first.
	^ ((self isValidMczName: name) and: [ (self directory fileExists: name) not ]) 
		ifTrue: [
			self directory newFileNamed: name do: [ :file |
				file binary.
				request entity writeOn: file ].
			ZnResponse created: request uri ]
		ifFalse: [ ZnResponse badRequest: request ]! !

!ZnMonticelloServerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 9/17/2010 14:28'!
handleRequest: request
	(request method = #GET)
		ifTrue: [
			^ (ZnUtils isSlash: request uri)
				ifTrue: [ self handleListMczEntries: request ]
				ifFalse: [ self handleGetMczEntry: request ] ].
	^ (request method = #PUT) 
		ifTrue: [ self handlePutMczEntry: request ]
		ifFalse: [ ZnResponse badRequest: request ]! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2010 10:24'!
isValidMczName: string
	^ string isString 
		and: [ string isEmpty not
			and: [ (string endsWith: '.mcz')
				and: [ (string includes: $/) not ] ] ]! !

!ZnMonticelloServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 10:21'!
mczEntries
	^ (self directory fileNames select: [ :each | each endsWith: '.mcz' ])
		asSortedCollection! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2010 11:00'!
repositoryListing
	^ String streamContents: [ :str |
		str nextPutAll: '<html><head><title>Monticello Repository</title></head>'.
		str nextPutAll: '<body><h1>Monticello Repository</h1>'.
		str nextPutAll: '<ul>'.
		self mczEntries do: [ :each | 
			str nextPutAll: '<li><a href="'; nextPutAll: each; nextPutAll: '">'; nextPutAll: each; nextPutAll: '</a></li>' ].
		str nextPutAll: '</ul></body></html>' ]! !

Object subclass: #ZnRequestLine
	instanceVariableNames: 'method uri version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnRequestLine commentStamp: '<historical>' prior: 0!
I am an HTTP Request Line.
I consist of an HTTP method, an URI and an HTTP protocol version.
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnRequestLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 15:18'!
method: method uri: uri
	^ self new
		method: method;
		uri: uri;
		version: ZnConstants defaultHTTPVersion;
		yourself ! !

!ZnRequestLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 13:50'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 13:45'!
method
	^ method! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:25'!
method: string
	method := ZnConstants knownHTTPMethods 
		detect: [ :each | each = string ] 
		ifNone: [ self error: 'Unknown HTTP method: ', string ] ! !

!ZnRequestLine methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/20/2010 19:06'!
printOn: stream
	super printOn: stream.
	(self method notNil and: [self uri notNil])
		ifTrue: [
			stream nextPut: $(; nextPutAll: self method; space.
			ZnUtils writeUrlPathQueryFragmentOf: self uri on: stream.
			stream nextPut: $) ]! !

!ZnRequestLine methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 19:25'!
readFrom: stream
	| line lineStream |
	line := (ZnLineReader on: stream) nextLine.
	lineStream := line readStream.
	self method: (lineStream upTo: Character space).
	self uri: (lineStream upTo: Character space).
	self version: (lineStream upToEnd)! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 13:45'!
uri
	^ uri! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 14:09'!
uri: anURI
	"We need an URI here, but the (Http)Url class is closer to what we need"
	
	uri := anURI asUrl! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 14:57'!
uriPathQueryFragment
	^ String streamContents: [ :stream | 
		ZnUtils writeUrlPathQueryFragmentOf: self uri on: stream ]! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 13:45'!
version
	^ version! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:25'!
version: string
	version := ZnConstants knownHTTPVersions 
		detect: [ :each | each = string ] 
		ifNone: [ self error: 'Unknown HTTP version: ', string ] ! !

!ZnRequestLine methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:40'!
writeOn: stream
	| writeStream |
	writeStream := ZnBivalentWriteStream on: stream.
	writeStream nextPutAll: self method; space.
	ZnUtils writeUrlPathQueryFragmentOf: self uri on: writeStream.
	writeStream space; nextPutAll: self version; nextPutAll: String crlf ! !

Object subclass: #ZnServer
	instanceVariableNames: 'port process serverSocket delegate authenticator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

ZnServer class
	instanceVariableNames: 'Default'!

!ZnServer commentStamp: 'SvenVanCaekenberghe 12/7/2010 13:15' prior: 0!
I am ZnServer.
I implement the server part of the HTTP protocol.
I can be customized with a delegate (#handleRequest:) and 
an authenticator (#authenticateRequest:do:).

ZnServer startDefaultOn: 1701.
ZnServer default authenticator: (ZnBasicAuthenticator username: 'foo' password: 'secret').
ZnClient get: 'http://localhost:1701' username: 'foo' password: 'secret'.

We only handle one shot responses for now.
Acknowledgement: basic design copied from Blackfoot.

Part of Zinc HTTP Components.!

ZnServer class
	instanceVariableNames: 'Default'!

ZnServer subclass: #ZnExperimentalServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnExperimentalServer commentStamp: 'SvenVanCaekenberghe 12/7/2010 13:13' prior: 0!
I am ZnExperimentalServer.
I am a ZnServer that keeps connections alive.

Part of Zinc HTTP Components.!

!ZnExperimentalServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 23:17'!
listenLoop
	"We create a listening Socket, then wait for a connection.
	After each connection we also check that the listening Socket is still valid 
	- if not we just make a recursive call to this method to start over."

	serverSocket := ZnUtils serverSocketOn: port.
	[ [ 
		serverSocket isValid ifFalse: [
			"will trigger #ifCurtailed: block and destroy socket"
			^ self listenLoop ].
		self serveConnectionsOn: serverSocket ] repeat ]

		ifCurtailed: [
			(Delay forMilliseconds: 10) wait.
			serverSocket destroy.
			serverSocket := nil ]! !

!ZnExperimentalServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 23:17'!
readRequestSafely: stream
	^ [ [ self readRequest: stream ]
		on: ConnectionClosed 
		do: [ nil ] ]
			on: ConnectionTimedOut 
			do: [ nil ]! !

!ZnExperimentalServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 23:17'!
serveConnectionsOn: listeningSocket
	"We wait up to 10 seconds for an incoming connection.
	If we get one we wrap it in a SocketStream and then we process it in three steps: 
	#readRequest: #handleRequest: and #writeResponse:on: in a loop"
		 
	| stream socket request response |
	socket := (listeningSocket waitForAcceptFor: 10) ifNil: [ ^ self ].
	stream := SocketStream on: socket.
	[ [ [ [
		(request := self readRequestSafely: stream) notNil
			ifTrue: [
				response := self handleRequest: request.
	 			self writeResponse: response on: stream ].
		request isNil or: [ request isConnectionClose ] ] whileFalse ]
	
		ensure: [ stream close ] ]
			ifCurtailed: [ socket destroy ] ]
				forkAt: Processor highIOPriority
				named: 'Zinc HTTP Server Worker'! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/7/2010 13:19'!
default
	"Return the Default instance that we manage"
	
	^ Default! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/7/2010 13:17'!
defaultOn: aNumber
	"Create a new Default instance on a given port,
	stopping any previously running Default and replacing it."

	Default isNil ifFalse: [ Default stop ].
	^ Default := self on: aNumber! !

!ZnServer class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 9/7/2010 22:58'!
initialize
	Smalltalk addToStartUpList: self after: AutoStart.
	Smalltalk addToShutDownList: self! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 9/7/2010 22:59'!
on: aNumber
	"Instantiate a new listener on a given port,
	send #start to it to start listening."

	^ self new port: aNumber; yourself! !

!ZnServer class methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 12/7/2010 13:17'!
shutDown
	"Stop the Default instance, if any."
	
	Default isNil ifFalse: [ Default stop ]! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/7/2010 13:16'!
startDefaultOn: aNumber
	"Start the Default instance on a given port,
	stopping any previously running Default."

	^ (self defaultOn: aNumber) 
		start; 
		yourself! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 9/7/2010 23:01'!
startOn: aNumber
	"Instantiate a new listener on a given port and start listening."

	^ (self on: aNumber) 
		start;
		yourself! !

!ZnServer class methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 12/7/2010 13:19'!
startUp
	"Start the Default instance, if any."
	
	Default isNil ifFalse: [ Default start ]! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/7/2010 13:18'!
stopDefault
	"Stop the Default instance, if any, and clear it."
	
	self shutDown.
	Default := nil! !

!ZnServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/30/2010 15:50'!
authenticateRequest: request do: block
	^ self authenticator isNil
		ifTrue: [ block value ]
		ifFalse: [ self authenticator authenticateRequest: request do: block ]! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2010 12:32'!
authenticator
	^ authenticator ! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 15:49'!
authenticator: object
	"When not nil, object will be sent #authenticateRequest:do:
	to authenticate or refuse the requests.
	When authentication succeeds, the block should be executed,
	when authentication fails, a ZnResponse should be returned"

	authenticator := object! !

!ZnServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/3/2010 14:03'!
defaultHandleRequest: request
	"This handler will run when there is no delegate."
	
	request uri path first = 'echo'
		ifTrue: [ ^ self echoRequest: request ].
	(ZnUtils isSlash: request uri)
		ifTrue: [ ^ ZnResponse ok: (ZnEntity html: ZnConstants welcomePageHtml) ].
	^ ZnResponse notFound: request uri! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/9/2010 20:45'!
delegate
	"Return the optional delegate, 
	the object that will be sent #value to handle a request and produce a response.
	If there is no delegate, #defaultHandleRequest will run."
	
	^ delegate! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 13:14'!
delegate: object
	"Set the delegate to object. 
	This will be sent #handleRequest: to handle a request and produce a response"
	
	delegate := object! !

!ZnServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/3/2010 14:04'!
echoRequest: request	
	| info entity |
	info := self generateEchoRequestString: request.
	entity := ZnEntity text: info.
	^ ZnResponse ok: entity! !

!ZnServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/6/2010 14:52'!
generateEchoRequestString: request
	"Generate a string echoing the request"
	^ String streamContents: [ :stream |
		stream nextPutAll: 'Hello World from Zinc HTTP Components !!'; crlf.
		stream nextPutAll: request method; nextPutAll: ' request for '.
		ZnUtils writeUrlPathQueryFragmentOf: request uri on: stream.
		stream crlf; nextPutAll: 'with headers'; crlf.
		request headersDo: [ :key :value |
			stream space; nextPutAll: key; nextPutAll: ': '; nextPutAll: value; crlf ].
		request entity isNil ifFalse: [
			stream nextPutAll: ' containing '; print: request entity ] ]! !

!ZnServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/30/2010 16:58'!
handleRequest: request
	^ self authenticateRequest: request do: [
		self delegate isNil
			ifTrue: [ self defaultHandleRequest: request ] 
			ifFalse: [ self delegate handleRequest: request ] ]! !

!ZnServer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/15/2010 09:34'!
isRunning
	^ self process notNil and: [ self serverSocket notNil ]! !

!ZnServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/15/2010 09:32'!
listenLoop
	"We create a listening Socket, then wait for a connection.
	After each connection we also check that the listening Socket is still valid 
	- if not we just make a recursive call to this method to start over."

	serverSocket := ZnUtils serverSocketOn: port.
	[ [ 
		serverSocket isValid ifFalse: [
			"will trigger #ifCurtailed: block and destroy socket"
			^ self listenLoop ].
		self serveConnectionOn: serverSocket ] repeat ]

		ifCurtailed: [
			(Delay forMilliseconds: 10) wait.
			serverSocket destroy.
			serverSocket := nil ]! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 22:55'!
port
	^ port! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 22:56'!
port: integer
	port := integer! !

!ZnServer methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/20/2010 20:54'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	stream nextPutAll: (self isRunning ifTrue: [ 'running'] ifFalse: [ 'stopped' ]).
	self port notNil ifTrue: [ stream space; print: self port ].
	stream nextPut: $)! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 22:56'!
process
	^ process! !

!ZnServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/21/2010 16:00'!
readRequest: stream
	| request |
	request := ZnRequest readFrom: stream.
	request headers 
		at: ZnConstants remoteAddressHeader
		put: (ZnUtils ipAddressToString: stream socket remoteAddress).
	^ request! !

!ZnServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/3/2010 15:27'!
serveConnectionOn: listeningSocket
	"We wait up to 10 seconds for an incoming connection.
	If we get one we wrap it in a SocketStream and then we process it in three steps: 
	#readRequest: #handleRequest: and #writeResponse:on: after which we close it"
		 
	| stream socket request response |
	socket := (listeningSocket waitForAcceptFor: 10) ifNil: [ ^ self ].
	stream := ZnUtils socketStreamOn: socket.
	[ [ [
		request := self readRequest: stream.
		(response := self handleRequest: request) setConnectionClose.
	 	self writeResponse: response on: stream ]
	
		ensure: [ stream close ] ]
			ifCurtailed: [ socket destroy ] ]
				forkAt: Processor highIOPriority
				named: 'Zinc HTTP Server Worker'! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 09:32'!
serverSocket
	^ serverSocket! !

!ZnServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 9/10/2010 11:52'!
start
	self stop.
	process := [ [ self listenLoop ] repeat ]
		forkAt: Processor highIOPriority 
		named: 'Zinc HTTP Server ', self port printString! !

!ZnServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 9/17/2010 16:23'!
stop
	self isRunning ifTrue: [
		process terminate. 
		process := nil ].
	^ self! !

!ZnServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/8/2010 09:43'!
writeResponse: response on: stream
	response writeOn: stream.
	stream flush! !

Object subclass: #ZnStaticFileServerDelegate
	instanceVariableNames: 'prefix directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnStaticFileServerDelegate commentStamp: '<historical>' prior: 0!
I am ZnStaticFileServerDelegate.
I am a simple proof of concept implementation of a web server serving static files.
I handle urls with prefix as requests for files in directory.
I function as a delegate for ZnServer.

ZnServer startDefaultOn: 1701.
ZnServer default delegate: ((ZnStaticFileServerDelegate new) 
									prefixFromString: 'apple/macoxs'; 
									directory: (FileDirectory on: '/Library/WebServer/Documents'); 
									yourself).

Part of Zinc HTTP Components.!

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2010 17:11'!
actualFilenameFor: uri
	| subElements subDir entry |
	(uri path beginsWith: self prefix) ifFalse: [ ^ nil ].
	subElements := (uri path allButFirst: self prefix size) reject: [ :each | each isEmpty ].
	subDir := subElements allButLast inject: self directory into: [ :parent :sub |
		(parent directoryExists: sub)
			ifTrue: [ parent directoryNamed: sub ]
			ifFalse: [ ^ nil ] ]. 
	subElements isEmpty
		ifTrue: [ entry := subDir containingDirectory directoryEntryFor: subDir localName ] 
		ifFalse: [ (entry := subDir directoryEntryFor: subElements last) isNil
					ifTrue: [ ^ nil ] ].
	^ entry isDirectory 
		ifTrue: [ 
			subDir := entry asFileDirectory.
			(subDir fileExists: 'index.html')
				ifTrue: [ (subDir directoryEntryFor: 'index.html') fullName ] 
				ifFalse: [ nil ] ]
		ifFalse: [ entry fullName ]! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/16/2010 12:19'!
directory
	^ directory ! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/16/2010 12:47'!
directory: fileDirectory
	directory := fileDirectory! !

!ZnStaticFileServerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 9/16/2010 15:13'!
handleRequest: request
	| actualFilename |
	actualFilename := self actualFilenameFor: request uri.
	^ actualFilename notNil
		ifTrue: [ self responseForFile: actualFilename ] 
		ifFalse: [ ZnResponse notFound: request uri ]! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/16/2010 12:19'!
prefix
	^ prefix  ! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/16/2010 12:21'!
prefix: orderedCollection
	prefix := orderedCollection! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 16:26'!
prefixFromString: string
	self prefix: (string findTokens: '/')! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/16/2010 16:34'!
responseForFile: filename
	| entry size time mimeType fileStream entity response |
	entry := FileDirectory root directoryEntryFor: filename.
	size := entry fileSize.
	time := entry modificationDateAndTime.
	mimeType := ZnMimeType forFilenameExtension: (FileDirectory extensionFor: filename).
	fileStream := FileDirectory root oldFileNamed: filename.
	mimeType isBinary ifTrue: [ fileStream binary ].
	(entity := ZnStreamingEntity type: mimeType length: size)
		stream: fileStream.
	(response := ZnResponse ok: entity)
		headers at: 'Modification-Date' put: (ZnUtils httpDate: time).
	^ response ! !

Object subclass: #ZnStatusLine
	instanceVariableNames: 'version code reason'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnStatusLine commentStamp: '<historical>' prior: 0!
I am an HTTP Status Line.
I consist of an HTTP protocol version, an integer HTTP status code and an HTTP reason string.
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 15:48'!
code: code
	^ self new
		code: code;
		version: ZnConstants defaultHTTPVersion;
		yourself! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 15:47'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:49'!
code
	^ code! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/20/2010 17:12'!
code: integer
	(ZnConstants httpStatusCodes includesKey: integer) ifFalse: [ 
		self error: 'Unknown HTTP status code: ', integer printString ].
	code := integer! !

!ZnStatusLine methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/20/2010 19:01'!
printOn: stream
	super printOn: stream.
	self code notNil
		ifTrue: [
			stream nextPut: $(; print: self code; space; nextPutAll: self reason; nextPut: $) ]! !

!ZnStatusLine methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 19:25'!
readFrom: stream
	| line lineStream |
	line := (ZnLineReader on: stream) nextLine.
	lineStream := line readStream.
	self version: (lineStream upTo: Character space).
	self code: (Integer readFrom: (lineStream upTo: Character space)).
	self reason: (lineStream upToEnd)
! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/20/2010 17:27'!
reason
	reason isNil 
		ifTrue: [ 
			reason := ZnConstants httpStatusCodes at: self code ifAbsent: [ 'Unknown Status Code' ] ].
	^ reason! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 16:29'!
reason: string
	reason := string! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:46'!
version
	^ version! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:46'!
version: string
	version := ZnConstants knownHTTPVersions 
		detect: [ :each | each = string ] 
		ifNone: [ self error: 'Unknown HTTP version: ', string ] ! !

!ZnStatusLine methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:40'!
writeOn: stream
	| writeStream |
	writeStream := ZnBivalentWriteStream on: stream.
	writeStream nextPutAll: self version; space.
	self code printOn: writeStream.
	writeStream space; nextPutAll: self reason; nextPutAll: String crlf ! !

Object subclass: #ZnUserAgent
	instanceVariableNames: 'session settings request response progressObservers errorHandler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnUserAgent commentStamp: '<historical>' prior: 0!
I implement a client for making HTTP requests. I can maintain session state across multiple requests, automatically follow redirects and notify observers of my request and response activity. The contents of my request headers can be modified easily.

Much of my behavior can be controlled with an instance of ZnUserAgentSettings.

Some examples of my use:

client := ZnUserAgent new.
response := client get: 'pharo-project.org'.
args := (Dictionary new) at: 'foo' put: 'bar'; yourself.
response := client post: 'mytestdomain.com/form' data: args.!

ZnUserAgent subclass: #ZnHttpClient
	instanceVariableNames: 'url params headers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnHttpClient commentStamp: '<historical>' prior: 0!
I implement an HTTP client with a high level API.

Some examples of my use:

response := ZnHttpClient new
	url: 'somedomain.com';
	auth: 'joe' password: 'secret';
	parameterAt: 'name' put: 'joe';
	headerAt: 'X-Custom Header' put: 'woot';
	get.
	 !

!ZnHttpClient methodsFor: 'private' stamp: 'MattKennedy 10/1/2010 20:39'!
argStringUnencoded: args
	"Return the args in a long string, as encoded in a url"

	| argsString first |
	args isString ifTrue: ["sent in as a string, not a dictionary"
		^ (args first = $? ifTrue: [''] ifFalse: ['?']), args].
	argsString := String new writeStream.
	argsString nextPut: $?.
	first := true.
	args associationsDo: [ :assoc |
		assoc value do: [ :value |
			first ifTrue: [ first := false ] ifFalse: [ argsString nextPut: $& ].
			argsString nextPutAll: assoc key.
			argsString nextPut: $=.
			argsString nextPutAll: value. ] ].
	^ argsString contents! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:36'!
auth: username password: password
	| cred |
	cred := ZnBasicCredential new
				username: username;
				password: password;
				domain: self url authority;
				yourself.
	self session addCredential: cred! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:23'!
contentLength
	^self response headers contentLength! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:23'!
contentType
	^self response headers contentType! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 21:20'!
cookieAt: aName
	^(super cookieAt: aName) value! !

!ZnHttpClient methodsFor: 'operations' stamp: 'MattKennedy 10/1/2010 22:03'!
delete
	self 
		sendMethod: [self method: #DELETE for: self url headers: self headers data: self params];
		resetParameters.
	^self response entity isNil ifTrue: [nil] ifFalse: [self response entity contents]! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:36'!
digestAuth: username password: password
	| cred |
	cred := ZnDigestCredential new
				username: username;
				password: password;
				domain: self url authority;
				yourself.
	self session addCredential: cred! !

!ZnHttpClient methodsFor: 'operations' stamp: 'MattKennedy 10/5/2010 17:32'!
get
	| fullUrl |
	fullUrl := (self url asString, (self argStringUnencoded: self params)) asUrl.
	self 
		sendMethod: [self method: #GET for: fullUrl headers: self headers data: nil];
		resetParameters.
	^self response entity isNil ifTrue: [nil] ifFalse: [self response entity contents]! !

!ZnHttpClient methodsFor: 'operations' stamp: 'MattKennedy 10/1/2010 22:02'!
head
	| fullUrl |
	fullUrl := (self url asString, (self argStringUnencoded: self params)) asUrl.
	self 
		sendMethod: [self method: #HEAD for: fullUrl headers: self headers data: nil];
		resetParameters.
	^self response entity isNil ifTrue: [nil] ifFalse: [self response entity contents]! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:17'!
headerAt: aName put: aValue
	self headers at: aName put: aValue! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:17'!
headers
	^ headers ifNil: [headers := Dictionary new]! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:10'!
headers: anObject
	headers := anObject! !

!ZnHttpClient methodsFor: 'testing' stamp: 'MattKennedy 10/1/2010 21:50'!
isAuthRequired
	^self isAuthRequired: self status! !

!ZnHttpClient methodsFor: 'private' stamp: 'MattKennedy 10/1/2010 21:55'!
linkCredentials
	(self session credentialsForUrl: self url) do: [:cred |
		cred linkToAuthRequest: (self response headers at: 'WWW-Authenticate')]! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:13'!
parameterAt: aName
	^self params at: aName ifAbsent: [nil]! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/4/2010 10:30'!
parameterAt: aName add: aValue
	| existingValue newValue |
	existingValue := self params at: aName ifAbsent: [nil].
	newValue := existingValue isNil
		ifTrue: [aValue]
		ifFalse: [
			existingValue isArray
				ifTrue: [existingValue copyWith: aValue]
				ifFalse: [Array with: existingValue with: aValue]].
	self params at: aName put: newValue ! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:15'!
parameterAt: aName put: aValue
	self params at: aName put: aValue! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:12'!
params
	^ params ifNil: [params := Dictionary new]! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:10'!
params: anObject
	params := anObject! !

!ZnHttpClient methodsFor: 'operations' stamp: 'MattKennedy 10/1/2010 22:02'!
post
	self 
		sendMethod: [self method: #POST for: self url headers: self headers data: self params];
		resetParameters.
	^self response entity isNil ifTrue: [nil] ifFalse: [self response entity contents]! !

!ZnHttpClient methodsFor: 'operations' stamp: 'MattKennedy 10/1/2010 22:03'!
put
	self 
		sendMethod: [self method: #PUT for: self url headers: self headers data: self params];
		resetParameters.
	^self response entity isNil ifTrue: [nil] ifFalse: [self response entity contents]! !

!ZnHttpClient methodsFor: 'private' stamp: 'MattKennedy 10/1/2010 21:58'!
resetParameters
	params := nil! !

!ZnHttpClient methodsFor: 'private' stamp: 'MattKennedy 10/4/2010 17:45'!
sendMethod: aBlock
	aBlock on: Error do: self errorHandler.
	self isAuthRequired ifTrue: [
		self linkCredentials.
		aBlock on: Error do: self errorHandler].
	self resetParameters ! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:47'!
status
	^self response statusLine code! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:10'!
url
	^ url! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:36'!
url: anObject
	url := anObject asUrl! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:12'!
userAgent
	^self settings agent! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:12'!
userAgent: aString
	self settings agent: aString! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:14'!
cookieAt: aName
	"Answers the named cookie for the current request or nil if non exists."
	^self session cookiejar cookieAt: aName forUrl: self request url! !

!ZnUserAgent methodsFor: 'private' stamp: 'MattKennedy 10/6/2010 17:20'!
defaultErrorHandler
	^[:error | self logProgress: error description, String crlf. error defaultAction ]! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 15:02'!
delete: urlObject
	^self delete: urlObject headers: nil! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 15:02'!
delete: urlObject headers: headerCollection
	^self method: #DELETE for: urlObject headers: headerCollection data: nil ! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 13:57'!
errorHandler
	^errorHandler ifNil: [errorHandler := self defaultErrorHandler]! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 13:45'!
errorHandler: aBlock
	errorHandler := aBlock! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:06'!
get: urlObject
	^self get: urlObject headers: nil! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:06'!
get: urlObject headers: headerCollection
	^self method: #GET for: urlObject headers: headerCollection data: nil ! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:11'!
head: urlObject
	^self head: urlObject headers: nil! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:10'!
head: urlObject headers: headerCollection
	^self method: #HEAD for: urlObject headers: headerCollection data: nil ! !

!ZnUserAgent methodsFor: 'testing' stamp: 'MattKennedy 9/28/2010 17:25'!
isAuthRequired: code
	(code = 401) ifTrue: [^true].
	^false! !

!ZnUserAgent methodsFor: 'testing' stamp: 'MattKennedy 9/27/2010 13:38'!
isRedirect: code
	(code = 302 or: [ code = 301]) ifTrue: [^true].
	^false! !

!ZnUserAgent methodsFor: 'private' stamp: 'MattKennedy 9/27/2010 13:43'!
logMessage: messageObject
	self progressObservers do: [:each | messageObject writeOn: each. each flush].
! !

!ZnUserAgent methodsFor: 'private' stamp: 'MattKennedy 9/27/2010 10:43'!
logProgress: aString
	self progressObservers do: [:each | each show: aString].
! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 10:44'!
logProgressToTranscript
	self progressObservers add: Transcript! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'MattKennedy 9/27/2010 13:34'!
method: aMethod for: urlObject headers: headerCollection data: dataEntity
	| limit |
	limit := self settings followRedirect ifTrue: [self settings redirectLimit] ifFalse: [1].
	^self method: aMethod for: urlObject headers: headerCollection data: dataEntity limit: limit.
	! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'MattKennedy 10/1/2010 14:14'!
method: aMethod for: urlObject headers: headerCollection data: dataEntity limit: aLimit
	| stream attempts url |
	attempts := 0.
	url := urlObject.
	[[ attempts < aLimit ] whileTrue: [
			self prepareRequest: url method: aMethod headers: headerCollection data: dataEntity.
			stream := self openConnection.
			request setConnectionClose.
			[ self sendRequest: stream ] ensure: [ stream close ].
			(self isRedirect: self response status)
				ifTrue: [ url := (self response headers at: 'Location') asUrl. attempts := attempts + 1]
				ifFalse: [attempts := aLimit]]]
			on: Error
			do: self errorHandler.
	^self response! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'MattKennedy 10/1/2010 14:03'!
openConnection
	^ZnUtils socketStreamToUrl: request url! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:07'!
post: urlObject data: dataEntity
	^self post: urlObject headers: nil data: dataEntity! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:07'!
post: urlObject headers: headerCollection data: dataEntity
	^self method: #POST for: urlObject headers: headerCollection data: dataEntity! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'MattKennedy 10/6/2010 17:20'!
prepareCredentials: headers for: urlObject method: method
	(self session credentialsForUrl: urlObject) do: [:cred |
		cred isComplete ifTrue: [
			[cred setAuthorizationHeader: headers for: urlObject method: method]
				on: Error
				do: self errorHandler]]
	! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'MattKennedy 9/28/2010 16:30'!
prepareData: dataEntity
	"Prepare the ZnEntity for the request."
	dataEntity ifNil: [^dataEntity ].
	(dataEntity isKindOf: ZnEntity) ifTrue: [^dataEntity].
	^ZnApplicationFormUrlEncodedEntity withAll: dataEntity! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 10/1/2010 21:35'!
prepareHeaders: headerCollection for: urlObject method: method
	| headers |
	headers := (ZnHeaders requestHeadersFor: urlObject)
					at: 'User-Agent' put: self settings agent;
					yourself.
	headerCollection isNil 
		ifFalse: [ headers addAll: headerCollection ].
	(self session cookiejar cookiesForUrl: urlObject) do: [:cookie |
		headers at: 'Cookie' add: cookie asString].
	self prepareCredentials: headers for: urlObject method: method.
	^headers
	! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'MattKennedy 9/26/2010 16:36'!
prepareRequest: urlObject method: method 
	^self prepareRequest: urlObject method:  method headers: nil! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'MattKennedy 9/26/2010 16:36'!
prepareRequest: urlObject method: method headers: headerCollection
	^self prepareRequest: urlObject method:  method headers: headerCollection data: nil! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'MattKennedy 9/29/2010 13:40'!
prepareRequest: urlObject method: method headers: headerCollection data: dataEntity
	| url |
	url := urlObject asUrl.
	request := ZnRequest new
		requestLine: (ZnRequestLine method: method uri: url);
		headers: (self prepareHeaders: headerCollection for: url method: method);
		entity: (self prepareData: dataEntity);
		yourself! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 12/6/2010 23:59'!
processResponse: stream
	| cookies |
	response := (request method = #HEAD) 
		ifTrue: [ZnResponse readHeaderFrom: stream]
		ifFalse: [ZnResponse readFrom: stream].
	self logMessage: response headers.
	cookies := response headers at: 'Set-Cookie' ifAbsent: [^response].
	cookies := (cookies isCollection & cookies isString) ifTrue: [Array with: cookies] ifFalse: [cookies].
	cookies do: [:cookie | 
		self session cookiejar add: (ZnCookie fromString: cookie for: self request url)].
	^response ! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 10:43'!
progressObservers
	progressObservers ifNil: [progressObservers := OrderedCollection new].
	^progressObservers! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:10'!
put: urlObject data: dataEntity
	^self put: urlObject headers: nil data: dataEntity! !

!ZnUserAgent methodsFor: 'operations' stamp: 'MattKennedy 9/27/2010 14:10'!
put: urlObject headers: headerCollection data: dataEntity
	^self method: #PUT for: urlObject headers: headerCollection data: dataEntity! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:24'!
request
	^ request! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:24'!
request: anObject
	request := anObject! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:24'!
response
	^ response! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:24'!
response: anObject
	response := anObject! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'MattKennedy 9/28/2010 13:24'!
sendRequest: stream
	self logMessage: request.
	request writeOn: stream.
	stream flush.
	^self processResponse: stream! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 11:17'!
session
	^ session ifNil: [ session := self settings loadSession ]! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:24'!
session: anObject
	session := anObject! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/26/2010 16:14'!
settings
	^ settings isNil 
		ifTrue: [settings := ZnUserAgentSettings defaultSettingsClass new]
		ifFalse: [settings]! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:24'!
settings: anObject
	settings := anObject! !

Object subclass: #ZnUserAgentSession
	instanceVariableNames: 'cookiejar credentials currentRealm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnUserAgentSession commentStamp: 'SvenVanCaekenberghe 12/6/2010 13:04' prior: 0!
I represent a session store for an HTTP client instance of ZnUserAgent. 

I hold the cookie jar for the session and any authentication credentials used by the client.

Part of Zinc HTTP Components.!

!ZnUserAgentSession class methodsFor: 'instance creation' stamp: 'MattKennedy 9/28/2010 10:52'!
withCookieJar: aCookieJar
	^self new cookiejar: aCookieJar; yourself! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:31'!
addCredential: aCredential
	self credentials at: aCredential domain, '@', aCredential realm put: aCredential ! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 00:00'!
cookiejar
	^ cookiejar ifNil: [ cookiejar := ZnCookieJar new ]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:50'!
cookiejar: anObject
	cookiejar := anObject! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:32'!
credentialAt: aRealm forUrl: urlObject
	^self credentials at: urlObject authority, '@', aRealm ifAbsent: [nil]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:30'!
credentials
	^ credentials ifNil: [credentials := Dictionary new]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:50'!
credentials: anObject
	credentials := anObject! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 10/6/2010 17:19'!
credentialsForUrl: urlObject
	^self credentials values select: [:cred | urlObject authority includesSubString: (cred domain)]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:24'!
currentRealm
	^ currentRealm! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:24'!
currentRealm: anObject
	currentRealm := anObject! !

Object subclass: #ZnUserAgentSettings
	instanceVariableNames: 'settings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnUserAgentSettings commentStamp: 'SvenVanCaekenberghe 12/6/2010 13:04' prior: 0!
I manage the settings for a ZnUserAgent instance. 

I can be subclassed to extend capabilities to include loading settings from a configuration store or maintaining persistent storage for a cookie jar.

Part of Zinc HTTP Components.!

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:53'!
commentString
	^'(', self platform platformName, '; ', self encryptionStrengthComment, '; ', self platformDetails, '; ', self localeComment, ')'! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 00:00'!
defaultCookieJar
	^ ZnCookieJar ! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:54'!
defaultSessionClass
	^ZnUserAgentSession! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'MattKennedy 9/26/2010 16:13'!
defaultSettingsClass
	^ZnUserAgentSettings ! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/1/2010 21:43'!
defaultUserAgentString
	^ZnConstants frameworkNameAndVersion, ' ', self commentString, ' ',  self platform version asString! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:51'!
encryptionStrengthComment
	"For user agent comment. Standard values are U, I, or N. Currently returning N until standardize https transport is available."
	^'N'! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:49'!
localeComment
	^LocaleID current isoString! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/6/2010 09:32'!
platform
	^ SmalltalkImage current os! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:51'!
platformDetails
	^self platform platformName, ' ', self platform platformSubtype, ' ', self platform osVersion! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:31'!
agent
	^ settings at: #agent ifAbsent: [nil]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:32'!
agent: anObject
	settings at: #agent put: anObject ! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:32'!
followRedirect
	^ settings at: #followRedirect ifAbsent: [nil]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:33'!
followRedirect: anObject
	settings at: #followRedirect put: anObject ! !

!ZnUserAgentSettings methodsFor: 'initialize-release' stamp: 'MattKennedy 9/26/2010 16:12'!
initialize
	super initialize.
	self settings at: #agent put: self class defaultUserAgentString.
	self followRedirect: true.
	self redirectLimit: 7.
	self timeout: 180! !

!ZnUserAgentSettings methodsFor: 'operations' stamp: 'MattKennedy 9/28/2010 10:55'!
loadSession
	^self class defaultSessionClass withCookieJar: self class defaultCookieJar new! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:33'!
redirectLimit
	^ settings at: #redirectLimit ifAbsent: [nil]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:33'!
redirectLimit: anObject
	settings at: #redirectLimit put: anObject ! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:31'!
settings
	^ settings isNil ifTrue: [ settings := Dictionary new] ifFalse: [settings]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:30'!
settings: anObject
	settings := anObject! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:33'!
timeout
	^ settings at: #timeout ifAbsent: [nil]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:34'!
timeout: anObject
	settings at: #timeout put: anObject ! !

Object subclass: #ZnUtils
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnUtils commentStamp: 'SvenVanCaekenberghe 12/6/2010 13:05' prior: 0!
ZnUtils holds various utilities most of which could be moves to *extension methods one day.

Part of Zinc HTTP Components.!

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 9/7/2010 14:59'!
asMimeType: object
	^ (object isKindOf: ZnMimeType)
		ifTrue: [ object ] 
		ifFalse: [ ZnMimeType fromString: object ] ! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 9/10/2010 20:14'!
capitalizeString: string
	| capitalExpected |
	capitalExpected := true.
	^ String new: string size streamContents: [ :stream |
		string do: [ :each |
			each isAlphaNumeric
				ifTrue: [
					capitalExpected 
						ifTrue: [ 
							stream nextPut: each asUppercase.
							capitalExpected := false. ]
						ifFalse: [ 
							stream nextPut: each asLowercase ] ] 
		 		ifFalse: [ 
					stream nextPut: each.
					capitalExpected := true ] ] ]! !

!ZnUtils class methodsFor: 'url-uri' stamp: 'SvenVanCaekenberghe 9/6/2010 21:24'!
hostAndPortOf: url
	^ String streamContents: [ :stream |
		stream nextPutAll: url authority.
		url port isNil ifFalse: [ stream nextPut: $:; print: url port ] ]! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 9/16/2010 15:52'!
httpDate
	^ self httpDate: DateAndTime now! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 9/17/2010 14:49'!
httpDate: dateAndTime
	"Answers a printer that formats dates for HTTP1.1 (RFC 1123). Eg. Sun, 06 Nov 1994 08:49:37 GMT"

	| utc |
	utc := dateAndTime asUTC.
	^ String streamContents: [ :stream |
		stream nextPutAll: utc dayOfWeekAbbreviation; nextPut: $,; space.
		stream nextPutAll: (utc dayOfMonth printStringLength: 2 padded: true).
		stream space; nextPutAll: utc monthAbbreviation; space.
		utc year printOn: stream.
		stream space.
		utc printHMSOn: stream.
		stream nextPutAll: ' GMT' ]! !

!ZnUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 9/21/2010 15:59'!
ipAddressToString: byteArray
	^ String streamContents: [ :stream |
			byteArray do: [ :each | stream print: each ] separatedBy: [ stream nextPut: $. ] ]! !

!ZnUtils class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:00'!
isCapitalizedString: string
	| capitalExpected |
	capitalExpected := true.
	string do: [ :each |
		each isAlphaNumeric
			ifTrue: [
				((capitalExpected and: [ each isUppercase])
					or: [ capitalExpected not and: [ each isLowercase] ])
					ifTrue: [ capitalExpected := false ]
					ifFalse: [ ^ false ] ] 
		 	ifFalse: [ capitalExpected := true ] ].
	^ true



! !

!ZnUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 9/12/2010 11:49'!
isProxySet
	^ HTTPSocket httpProxyServer isEmpty not! !

!ZnUtils class methodsFor: 'url-uri' stamp: 'SvenVanCaekenberghe 9/17/2010 14:26'!
isSlash: uri
	^ uri path size = 1 and: [ uri path first = '' ]! !

!ZnUtils class methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/27/2010 15:00'!
nextPutAll: collection on: stream chunked: size
	| written |
	written := 0.
	[ written < collection size ] whileTrue: [ | chunkSize |
		chunkSize := size min: (collection size - written).
		chunkSize printOn: stream base: 16.
		stream nextPutAll: String crlf.
		chunkSize timesRepeat: [ stream nextPut: (collection at: (written := written + 1)) ].
		stream nextPutAll: String crlf].
	stream nextPutAll: String crlf! !

!ZnUtils class methodsFor: 'url-uri' stamp: 'SvenVanCaekenberghe 9/11/2010 21:31'!
parseQueryFrom: stream
	| string index fields |
	fields := Dictionary new.
	[ stream atEnd ] whileFalse: [
		string := stream upTo: $&.
		index := string indexOf: $=.
		index > 0
			ifFalse: [ fields at: string unescapePercents put: nil ]
			ifTrue: [
				fields
					at: (string copyFrom: 1 to: index - 1) unescapePercents
					put: (string copyFrom: index + 1 to: string size) unescapePercents ] ].
	^ fields! !

!ZnUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 9/10/2010 11:52'!
serverSocketOn: port
	| socket |
	socket := Socket newTCP.
	self setSocketOptions: socket.
	socket listenOn: port backlogSize: 32.
	socket isValid
		ifFalse: [ self error: 'Cannot create socket on port ', port printString ].
	^ socket! !

!ZnUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 9/8/2010 08:17'!
setSocketOptions: socket
! !

!ZnUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 12/3/2010 15:27'!
socketStreamOn: socket
	| stream |
	stream := SocketStream on: socket.
	stream binary.
	^ stream! !

!ZnUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 9/12/2010 11:50'!
socketStreamToProxy
	^ SocketStream 
		openConnectionToHostNamed: HTTPSocket httpProxyServer 
		port: HTTPSocket httpProxyPort ! !

!ZnUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 10/14/2010 14:34'!
socketStreamToUrl: url
	| stream |
	stream := self isProxySet 
		ifTrue: [ self socketStreamToProxy ] 
		ifFalse: [ self socketStreamToUrlDirectly: url ].
	stream binary.
	^ stream! !

!ZnUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 9/12/2010 11:49'!
socketStreamToUrlDirectly: url
	| hostname port |
	hostname := url authority.
	port := url port isNil 
		ifTrue: [ 80 ] 
		ifFalse: [ url port ].
	^ SocketStream openConnectionToHostNamed: hostname port: port ! !

!ZnUtils class methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/17/2010 13:40'!
streamFrom: inputStream to: outputStream size: totalSize
	| bufferSize buffer leftToRead |
	bufferSize := 4096 min: totalSize.
	buffer := (inputStream isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]) new: bufferSize.
	leftToRead := totalSize.
	[ leftToRead > 0 ]
		whileTrue: [ | readCount |
			readCount := bufferSize min: leftToRead.
			inputStream next: readCount into: buffer.
			leftToRead := leftToRead - readCount.
			outputStream next: readCount putAll: buffer startingAt: 1 ]! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 10/5/2010 21:24'!
trimString: string
	"Until String>>#trimBoth is available everywhere"
	
	| left right |
	left := 1.
	right := string size.
	[ left <= right and: [ (string at: left) isSeparator ] ]
		whileTrue: [ left := left + 1 ].
	[ left <= right and: [ (string at: right) isSeparator ] ]
		whileTrue: [ right := right - 1 ].
	^ string copyFrom: left to: right! !

!ZnUtils class methodsFor: 'url-uri' stamp: 'SvenVanCaekenberghe 9/17/2010 15:33'!
urlPathQueryFragmentOf: url
	^ String streamContents: [ :stream | 
		self writeUrlPathQueryFragmentOf: url on: stream ]! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 9/13/2010 21:32'!
writeQueryFields: dictionary on: stream
	| first |
	first := true.
	dictionary keysAndValuesDo: [ :key :value |
		first ifFalse: [ stream nextPut: $& ].
		first := false.
		stream nextPutAll: key encodeForHTTP.
		value isNil ifFalse: [
			stream 
				nextPut: $=;
				nextPutAll: value encodeForHTTP ] ]! !

!ZnUtils class methodsFor: 'url-uri' stamp: 'SvenVanCaekenberghe 9/6/2010 14:50'!
writeUrlPathQueryFragmentOf: url on: stream
	url path do: [ :each |
		stream nextPut: $/; nextPutAll: each encodeForHTTP ].
	url query isNil ifFalse: [ 
		stream nextPut: $?; nextPutAll: url query ].
	url fragment isNil ifFalse: [
		stream nextPut: $#; nextPutAll: url fragment ]! !

Object subclass: #ZnValueDelegate
	instanceVariableNames: 'object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnValueDelegate commentStamp: 'SvenVanCaekenberghe 12/7/2010 16:14' prior: 0!
I am ZnValueDelegate.
I forward #handleRequest: messages to the object that I wrap using #value:

ZnServer default delegate: 
	(ZnValueDelegate with: [ :request | 
		ZnResponse ok: (ZnEntity with: 'You asked for ', request uri printString) ] ).

Part of Zinc HTTP Components.!

!ZnValueDelegate class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/7/2010 16:15'!
with: object
	^ (super basicNew) object: object; yourself! !

!ZnValueDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/7/2010 16:16'!
handleRequest: request
	^ object value: request! !

!ZnValueDelegate methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/7/2010 16:15'!
object: anObject
	object := anObject! !
ZnConstants initialize!
ZnMimeType initialize!
ZnServer initialize!
